//=========================================================================
//		NSBox+DynamicContent.m					TomatoTorrent
//=========================================================================
//  Created by Sarwat Khan on Thu Jun 12 2003.
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khans name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import "NSBox+DynamicContent.h"


@implementation NSBox (DynamicContent)
- switchContentViewTo:view showingViewDuringResize:empty;	//if empty is nil, uses contentview.
{
	id				window,
					oldView,
					rootView,
					superSuperview;		//sigh.
	NSRect			origFrame, 
					newFrame,
					windowFrame;
	float			origWindowHeight,
					origWindowWidth;
	BOOL			canDraw;
	unsigned int	savedAutosize, 
					superSuperSavedAutosize = 0;
	
	window = [self window];
	rootView = [window contentView];
	oldView = [self contentView];
	canDraw = [self canDraw];
	origFrame = [oldView frame];
	newFrame = [view frame];
	
	if (oldView == view) return self;	//nothing to do, bail out now.
	
	if (empty == nil) empty = [self contentView];
	
	savedAutosize = [[self superview] autoresizingMask];
	[[self superview] setAutoresizingMask:(NSViewWidthSizable | NSViewHeightSizable)];
	
	superSuperview = [[self superview] superview];
	if (superSuperview != rootView)
	{
		superSuperSavedAutosize = [superSuperview autoresizingMask];
		[superSuperview setAutoresizingMask:(NSViewWidthSizable | NSViewHeightSizable)];
	}
	
	if (empty != oldView)
		[self setContentView:empty];	//empty might be equal to view, in which case
										//we don't have to set it below.
			
	//grow the window to the size of newFrame.
	//have to adjust window frame size and window frame origin
	windowFrame = [window frame];

	origWindowHeight = windowFrame.size.height;
	windowFrame.size.height -= origFrame.size.height;
	windowFrame.size.height += newFrame.size.height;
	windowFrame.origin.y += origWindowHeight - windowFrame.size.height;

	origWindowWidth = windowFrame.size.width;
	windowFrame.size.width -= origFrame.size.width;
	windowFrame.size.width += newFrame.size.width;
	
	//actually do it
	[window setFrame:windowFrame display:canDraw animate:canDraw];
	
	if (empty != view) 	//we didn't set the given view
	{
		[self setContentView:view];
		[oldView setFrame:origFrame];
	}

	[[self superview] setAutoresizingMask:savedAutosize];
	if (superSuperview != rootView)
		[superSuperview setAutoresizingMask:superSuperSavedAutosize];

	return self;
}

@end
