//==============================================================================
//		sk_debug.c
//==============================================================================
//  Created by Sarwat Khan on Wed Jun 18 2003.
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khan’s name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include "sk_debug.h"

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <sys/syslog.h>

int 	g_sk_debug_level = -1;

bool 	g_sk_debug_use_syslog = false;
FILE*	g_sk_debug_file = NULL;

int 	
sk_debug_level()
{
	return g_sk_debug_level;
}

void 	
sk_debug_set_level(int level)
{
	g_sk_debug_level = level;
}

void 	
sk_debug_set_log_file(FILE *file)
{
	g_sk_debug_file = file;
}

FILE*	
sk_debug_log_file()
{
	return g_sk_debug_file;
}

bool	
sk_debug_use_syslog()
{
	return g_sk_debug_use_syslog;
}

void	
sk_debug_set_use_syslog(bool use)
{
	g_sk_debug_use_syslog = use;
}

void sk_debug_log(int level, char* in_format, ...)
{
	if (level >= g_sk_debug_level)
	{
		va_list		ap;
		
		if (g_sk_debug_use_syslog)
		{
			va_start(ap, in_format);
			vsyslog(LOG_DEBUG, in_format, ap);
			va_end(ap);
		}
		
		if (g_sk_debug_file)
		{
			unsigned	length = strlen(in_format);
			char		*format = in_format;
			bool		dispose = false;
			
			if (format[length-1] != '\n')
			{
				format = (char*)malloc(length+2);
				strcpy(format, in_format);
				format[length] = '\n';
				format[length+1] = 0;
				dispose = true;
			}
			
			va_start(ap, in_format);
			vfprintf(g_sk_debug_file, format, ap);
			va_end(ap);
			
			if (dispose) free(format);
		}
	}
}
