//==============================================================================
//		SKContainerFormatter.m					TomatoTorrent
//==============================================================================
//  Created by Sarwat Khan on Thu Jun 05 2003.
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khans name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import "SKContainerFormatter.h"
#import "SKLocalizedString.h"

@implementation SKContainerFormatter
- init
{
	_locale = [[[NSUserDefaults standardUserDefaults] dictionaryRepresentation] retain];
	return [super init];
}

- (void) dealloc
{
	[_locale release];
	[_summaryPlural release];
	[_summarySingular release];
	
	[super dealloc];
}

- setShowsSummary:(BOOL)showSummary singularTitle:singular pluralTitle:plural;
{
	if (showSummary)
	{
		[self setSingularTitleFormat:singular];
		[self setPluralTitleFormat:plural];
	}
	else
		[self setShowsSummary:NO];
	
	return self;
}

- setPluralTitleFormat:title
{
	[_summaryPlural release];
	_summaryPlural = [title copyWithZone:[self zone]];
	return self;
}

- setSingularTitleFormat:title
{
	[_summarySingular release];
	_summarySingular = [title copyWithZone:[self zone]];
	return self;
}

- pluralTitleFormat
{
	return _summaryPlural;
}

- singularTitleFormat
{
	return _summarySingular;
}

- setShowsSummary:(BOOL)set
{
	if (set)
	{
		if (_summarySingular == nil) [self setSingularTitleFormat:SKLocalizedString(@"%d item", nil)];
		if (_summaryPlural == nil) [self setPluralTitleFormat:SKLocalizedString(@"%d items", nil)];
	}
	else
	{
		[self setSingularTitleFormat:nil];
		[self setPluralTitleFormat:nil];
	}
	
	return self;
}

- (BOOL)showsSummary
{
	return _summarySingular != nil;
}

+ (NSString *)stringForObjectValue:(id)obj locale:locale singularTitle:singular pluralTitle:plural
{
	NSMutableString		*result = nil;
	id					defaultString(id obj) 	//I want BlockContexts...
						{ 
							id result = nil;
							
							if ([obj respondsToSelector:@selector(descriptionWithLocale:)])
								result = [obj descriptionWithLocale:locale];
							else
								result = [obj description];
							
							return result;
						}
	
	if ([obj respondsToSelector:@selector(count)])
	{
		if (singular == nil)
		{
			int		numReplaced;
			
			if ([obj respondsToSelector:@selector(descriptionWithLocale:indent:)])
				result = [NSMutableString stringWithString:
					[obj descriptionWithLocale:locale indent:1]];	//indent uses tabs.	//+++ apparently it doesn't.
			else
			{
				result = [NSMutableString stringWithString:defaultString(obj)];
			}
			
			//remove newlines. 10.2 >= only.
			if ([result respondsToSelector:@selector(replaceOccurrencesOfString:withString:options:range:)])
			{
				[result replaceOccurrencesOfString:@"\n" 
										withString:@"" 
										   options:0 
											 range:NSMakeRange(0, [result length])];
				
				numReplaced = [result replaceOccurrencesOfString:@"\t"
													  withString:@""
														 options:0 
														   range:NSMakeRange(0, [result length])];
	
				if (numReplaced == 0)
				{
					[result replaceOccurrencesOfString:@"  "
											withString:@""
											   options:0 
												 range:NSMakeRange(0, [result length])];
				}
			}
		}
		else
		{
			int		count = [obj count];
			
			if (count == 1)
			{
				result = [NSString stringWithFormat:singular, count];
			}
			else
			{
				result = [NSString stringWithFormat:plural, count];
			}
		}
	}
	else
		result = defaultString(obj);
	
	return result;
}

+ (NSString *)stringForObjectValue:(id)obj locale:locale;
{
	return [self stringForObjectValue:obj locale:locale singularTitle:nil pluralTitle:nil];
}

- (NSString *)stringForObjectValue:(id)obj;
{
	return [[self class] stringForObjectValue:obj 
									   locale:_locale 
								singularTitle:_summarySingular
								  pluralTitle:_summaryPlural];
}

@end
