//==============================================================================
//		• AppDelegate.h									TomatoTorrent
//==============================================================================
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khan’s name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import "AppDelegate.h"
#import "FetchURLWindowController.h"

#import "DownloadDocument.h"
#import "StartingPointController.h"
#import "TrackerDocument.h"
#import "TomatoTorrentAgent.h"
#import "GenerateTorrentsController.h"
#import "ReleaseNotesController.h"
#import "PreferencesController.h"

#import "SKDataFormatters.h"
#import "SKContainerFormatter.h"
#import "TorrentFaultFormatter.h"

#include "sk_debug.h"
#include "tomato_agent_logging.h"

#import "NSFileManager+ResourceDirectories.h"

#define kMegabyteFormatterKey		@"megabyte formatter key"
#define kByteFormatterKey			kTomatoAgentKey_generateState_totalBytes

static id g_formatters = nil;

@interface BitTorrentHoursFormatter : NSFormatter
@end

@implementation AppDelegate

- (void)applicationWillFinishLaunching:(NSNotification *)notification;
{
	//defaults for autostart seconds and debug level
	[[NSUserDefaults standardUserDefaults] registerDefaults:
		[NSDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithFloat:60],		kUserDefault_downloadAutostartSecs,

#if defined(__TomatoDebug_)			
			@"0",								@"ttagent_logging_level",
#else
			@"-1",								@"ttagent_logging_level",
#endif			
			nil]];

	//set debug level based on defaults
	sk_debug_set_level([[NSUserDefaults standardUserDefaults] integerForKey:@"ttagent_logging_level"]);
	ttagent_logging_set_level([[NSUserDefaults standardUserDefaults] integerForKey:@"ttagent_logging_level"]);

	// load defaults from the agent, which takes a few secs for Python to startup
	[NSThread detachNewThreadSelector:@selector(__loadBitTorrentDefaults:) toTarget:self withObject:nil];
	
	//services
	[NSApp setServicesProvider:self];
	
	//get starting point ready.
	[StartingPointController instance];
	
	//tell the release notes to show.
	[[ReleaseNotesController instance] performSelector:@selector(showWindowIfNewerVersion)
											withObject:nil
											afterDelay:0.42];	//get the starting point to show.
	
	__cachesFolderPath = [[NSFileManager defaultManager] 
		cachesDirectoryNamed:[[NSProcessInfo processInfo] processName] inDomain:NSUserDomainMask];
	[TomatoTorrentAgent setTorrentCachesDirectory:__cachesFolderPath];
}

- cachesDirectoryPathForCacheKey:(NSString*)key
{
	id		result = NSTemporaryDirectory();
	
	if (key)
	{
		BOOL		made = NO;
		id			fileManager = [NSFileManager defaultManager];
		id			new;
		
		new = [__cachesFolderPath stringByAppendingPathComponent:key];
		[fileManager createDirectoryAtPath:new attributes:nil];
		[fileManager fileExistsAtPath:new isDirectory:&made];
		
		if (made)
			result = new;
	}
	else
		result = __cachesFolderPath;
	
	return result;	
}

- (void) showStartingPoint:sender
{
	[[StartingPointController instance] showStartingPointWindow];
}

- (BOOL)applicationOpenUntitledFile:(NSApplication *)theApplication
{
	[self showStartingPoint:nil];
	return YES;
}

- (IBAction)openURL:(id)sender
{
	[[FetchURLWindowController instance] showWindow:sender];
}

- (IBAction) openUntitledTrackerDocument:sender;
{
	[[NSDocumentController sharedDocumentController] openUntitledDocumentOfType:kTrackerDocumentType
																		display:YES];	
}

- (IBAction)openTorrentGenerationWindow:(id)sender
{
	[[GenerateTorrentsController instance] showGenerationSetup:sender];
}

- (IBAction) openLastTrackerDocument:sender;
{
	id		url;
	
	if (url = [TrackerDocument lastTrackerDocumentURL])
		[[NSDocumentController sharedDocumentController] openDocumentWithContentsOfURL:url
																			   display:YES];
}

- (IBAction) openTrackerDocument:sender;
{
	id		panel = [NSOpenPanel openPanel];
	
	[panel setAllowsMultipleSelection:NO];
	[panel beginSheetForDirectory: NSHomeDirectory() 
							 file: nil
							types: [NSArray arrayWithObject:kTrackerDocumentType]
				   modalForWindow: nil 
				    modalDelegate: self 
				   didEndSelector: @selector(__trackerOpenPanelDidEnd:returnCode:contextInfo:) 
					  contextInfo: nil];
}

- (void) openReleaseNotes:sender
{
	[[ReleaseNotesController instance] showWindow:sender];
}

- (void) orderFrontPreferencesWindow:sender
{
	[[[PreferencesController sharedInstance] window] makeKeyAndOrderFront:sender];
}

- (void) openLicenseWithWorkspace:sender
{
	id		help = [[NSBundle mainBundle] pathForResource:@"License" ofType:@"html" inDirectory:@"Tomato Torrent Help/pages"];
	
	if (help)
	{
		[[NSWorkspace sharedWorkspace] openFile:help];
	}
	else
		NSBeep();
}

//---------------------------------------------------------------------------
//		• - endOfLineMenuItemValidation:
//---------------------------------------------------------------------------
//	Resets any menu item titles that can change to their default values.
//
//	For all others, we just return YES because that's what we want for the ones 
//	AppDelegate handles.

- (BOOL) validateMenuItem:(NSMenuItem*)item
{
	BOOL	result = YES;
	SEL		action = [item action];
	
	if (action == @selector(toggleTorrentRunningAction:))
	{
		[item setTitle:NSLocalizedString(@"Stop Torrent", @"stop torrent menu item")];
		result = NO;
	}
	
	return result;
}

- (void) toggleTorrentRunningAction:sender	
{
	//dud
}

- (void) doOpenURLService:(NSPasteboard *)pboard
				 userData:(NSString *)userData
					error:(NSString **)error
{
	id		string, types;
	
	types = [pboard types];
	
	if ([types containsObject:NSStringPboardType])
	{
		if (string = [pboard stringForType:NSStringPboardType])
		{
			[FetchURLWindowController fetchTorrentFromURLString:string];
		}
		else
			*error = SKLocalizedString(	@"Could not get text from the clipboard.", 
										@"service failed; couldn't get string from pboard");
	}
	else
		*error = SKLocalizedString(	@"Could not get text from the clipboard.", 
									@"service failed; couldn't get string from pboard");
}

- (void)__trackerOpenPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
	if (returnCode == NSOKButton)
	{
		[[NSDocumentController sharedDocumentController] openDocumentWithContentsOfURL:[[sheet URLs] lastObject] 
																			   display:YES];
	}
}

- (void) __loadBitTorrentDefaults:object
{
	id	pool, downloadDefaults;
	
	pool = [NSAutoreleasePool new];
		downloadDefaults = [TomatoTorrentAgent defaultDownloadConfigurationDictionary];
		[self performSelectorOnMainThread: @selector(__initializeDownloadDefaultsIfNeededWithObject:)
							   withObject: downloadDefaults
							waitUntilDone: NO];
		
		[TomatoTorrentAgent defaultTrackerOptions];
	[pool release];
}

- (void) __initializeDownloadDefaultsIfNeededWithObject:defaults
{
	//NSUserDefaults isn't thread safe (it isn't really multi-process safe either), so
	//the main thread will check and stash the given object if we don't have anything in
	//NSUserDefaults. This is just so that the Preferences window has something for the max
	//upload rate the first time it's brought up.
	id		userDefaults = [NSUserDefaults standardUserDefaults];
	
	if ([userDefaults objectForKey:kUserDefault_TomatoAgent_downloadOptions] == nil)
		[userDefaults setObject:defaults forKey:kUserDefault_TomatoAgent_downloadOptions];
	
	[PreferencesController noteTorrentDownloadOptionsDefaultsChanged];
}

- __initializeFormatters
{
	id		percentFormatter,
			localizedStringFormatter;
	id		rawSizeFormatter,
			megabyteSizeFormatter,
			pathFormatter,
			summaryContainerFormatter,
			booleanFormatter;
	
	/*
		The fact that this is global isn't the best setup as key names aren't that
		unique and can clash easily. However, there aren't any problems yet and
		it's convenient.
	 */
	
	percentFormatter 			= [[[SKFractionFormatter new] setMultiplierWithDouble:100] autorelease];
	localizedStringFormatter 	= [[SKLocalizedStringFormatter new] autorelease];
	rawSizeFormatter 			= [[SKDataSizeFormatter new] autorelease];
	megabyteSizeFormatter 		= [[SKDataSizeFormatter new] autorelease];
	pathFormatter 				= [[SKFilePathFormatter new] autorelease];
	summaryContainerFormatter 	= [[SKContainerFormatter new] autorelease];
	booleanFormatter			= [[SKBooleanFormatter new] autorelease];
	
	[percentFormatter setFormat:@"0.0#%"];
//	[localizedStringFormatter setTableName:kTomatoAgentStringsTable_downloadState];
//	[localizedStringFormatter setBundle:TTAgentBundle_()];
	[megabyteSizeFormatter formatMegabyteValues];
	[summaryContainerFormatter setShowsSummary:YES];
	
	g_formatters = [[NSDictionary alloc] initWithObjectsAndKeys:
		
		rawSizeFormatter,
		kTomatoAgentKey_downloadState_size,
		
		booleanFormatter,
		kTomatoAgentKey_metainfo_arg_isArchive,	//also metainfo.
		
		[[SKUnixTimeFormatter new] autorelease],
		kTomatoAgentKey_metainfo_arg_creationDate,	//also tracker
		
		localizedStringFormatter,
		kTomatoAgentKey_downloadState_status_activity,			//also tracker
		
		percentFormatter,
		kTomatoAgentKey_downloadState_status_fractionDone,
		
		[[BitTorrentHoursFormatter new] autorelease],
		kTomatoAgentKey_downloadState_status_hmsTimeEstimate,
		
		rawSizeFormatter,
		kTomatoAgentKey_downloadState_status_downTotal,

		rawSizeFormatter,
		kTomatoAgentKey_downloadState_status_upTotal,
		
		megabyteSizeFormatter,
		kMegabyteFormatterKey,
		
		summaryContainerFormatter,
		kTomatoAgentKey_downloadState_status,
/*
		[[[SKContainerFormatter new] setShowsSummary:YES 
									   singularTitle:SKLocalizedString(@"%d connection", nil)
									     pluralTitle:SKLocalizedString(@"%d connections", nil)] autorelease],
		kTomatoAgentKey_downloadState_stats_spew,
		
		booleanFormatter,
		kTomatoAgentKey_downloadState_stats_spew_unchoke,
*/		
		[[[SKContainerFormatter new] setShowsSummary:YES 
									   singularTitle:SKLocalizedString(@"%d error", nil)
									     pluralTitle:SKLocalizedString(@"%d errors", nil)] autorelease],
		kTomatoAgentKey_downloadState_errors,
		
		
		rawSizeFormatter,
		kTomatoAgentKey_generateState_totalBytes,
		/*
		rawSizeFormatter,
		kTomatoAgentKey_metainfo_info_length,
		
		rawSizeFormatter,
		kTomatoAgentKey_metainfo_info_files_length,
		*/
		
		pathFormatter,
		kTomatoAgentKey_downloadState_destPath,
		
		[[SKUnixTimeFormatter new] autorelease],
		kTomatoAgentKey_metainfo_arg_creationDate,
		
		[[SKContainerFormatter new] autorelease],
		@"__containerFormatter__",
		
		[[TorrentFaultFormatter new] autorelease],
		@"fault",
		
		nil];
		
	return self;
}

- (BOOL) __keyHasBooleanValue:key;
{
	static id downloadBools = nil;
	static id trackerBools = nil;
	//static id generateBools = nil;	//I don't use this.
	
	BOOL	result = NO;

	if (downloadBools == nil)
	{
		downloadBools = [TomatoTorrentAgent defaultDownloadConfigurationDictionary];
		trackerBools = [TomatoTorrentAgent defaultTrackerOptions];

		downloadBools = [downloadBools objectForKey:kTomatoAgentModuleDefaultsKey_boolKeys];
		trackerBools = [trackerBools objectForKey:kTomatoAgentModuleDefaultsKey_boolKeys];
	}
	
	result = [downloadBools containsObject:key];
	
	if (result == NO)
		result = [trackerBools containsObject:key];
		
	return result;

/*	static id booleanKeys = nil;
	
	if (booleanKeys == nil)
	{
		booleanKeys = [[NSSet alloc] initWithObjects:
						@"allow_get",
						@"parse_allowed_interval",
						@"show_names",
						@"keep_dead",
						@"nat_check",
						@"only_local_override_ip",
						
						@"check_hashes",
						nil];
	}
	
	return [booleanKeys member:key] != nil;*/
}

- stringForObjectValue:object forKey:key;
{
	id	formatter;
	id	result = nil;
	
	if (g_formatters == nil)
		[self __initializeFormatters];
	
	if (formatter = [g_formatters objectForKey:key])
	{
		result = [formatter stringForObjectValue:object];	
	}
	else if ([key isEqual:@"upRate"] | [key isEqual:@"upRate2"] || [key isEqual:@"downRate"])
	{
		result = [NSString stringWithFormat:SKLocalizedString(@"%2.1f K/s", @"transfer rate"),
			[object doubleValue]/1024];
	}
	else if ([self __keyHasBooleanValue:key])
	{
		result = [[[SKBooleanFormatter new] autorelease] stringForObjectValue:object];
	}
	else if ([object respondsToSelector:@selector(count)])
	{
		formatter = [g_formatters objectForKey:@"__containerFormatter__"];
		result = [formatter stringForObjectValue:object];
	}
	else
		result = [object description];
		
	return result;
}

- formatDataSize:object;
{
	return [(id)self stringForObjectValue:object forKey:kByteFormatterKey];
}

- formatMegabyteDataSize:object;
{
	return [(id)self stringForObjectValue:object forKey:kMegabyteFormatterKey];
}


@end
	
@implementation BitTorrentHoursFormatter

	//the progress controller doesn't use this formatter yet. The prog controller
	//also uses different format strings (they're shorter).

- (NSString *)stringForObjectValue:(id)object;
{
	id		result = object;
	
	if ([object respondsToSelector:@selector(objectAtIndex:)])
	{
		int		days, hours, minutes, seconds;
		id		format;
		id		days_s, hours_s, minutes_s, seconds_s;
		//id		hours_short_s, minutes_short_s, seconds_short_s;
		
		hours = [[object objectAtIndex:0] intValue];
		minutes = [[object objectAtIndex:1] intValue];
		seconds = [[object objectAtIndex:2] intValue];
		days = hours / 24;
		hours = hours % 24;
		
		days_s = SKLocalizedString((days != 1 ? @"days" : @"day"), nil);
		hours_s = SKLocalizedString((hours != 1 ? @"hours" : @"hour"), nil);
		minutes_s = SKLocalizedString((minutes != 1 ? @"minutes" : @"minute"), nil);
		seconds_s = SKLocalizedString((seconds != 1 ? @"seconds" : @"second"), nil);

		//hours_short_s = SKLocalizedString((hours > 1 ? @"hrs" : @"h"), nil);
		//minutes_short_s = SKLocalizedString((minutes > 1 ? @"minutes_short" : @"minute_short"), nil);
		//seconds_short_s = SKLocalizedString((seconds > 1 ? @"secs" : @"sec"), nil);		
		
		if (days >= 1)
		{
			format = SKLocalizedString(@"%d %@ %d %@", @"d h");
			result = [NSString stringWithFormat:format, days, days_s, hours, hours_s];
		}
		else if (hours > 0)
		{
			format = SKLocalizedString(@"%d %@ %d %@", @"h m s");
			result = [NSString stringWithFormat:format, hours, hours_s, minutes, minutes_s];
		}
		else
		{
			format = SKLocalizedString(@"%d %@ %d %@", @"m s");
			result = [NSString stringWithFormat:format, minutes, minutes_s, seconds, seconds_s];
		}
	}
	
	return result;
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error
{
	return NO;
}
@end
