//
//  DownloadDocument.m
//  TomatoTorrent
//
//  Created by Sarwat Khan on Mon Jun 02 2003.
//  Copyright (c) 2005, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khan’s name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "DownloadDocument.h"
#import "TomatoTorrentAgent.h"
#import "SKInternetConfig.h"

#import "DownloadProgressController.h"
#import "DownloadSetupWindowController.h"

//constants

#define kUserDefault_downloadDoc_saveDownloadPaths	@"DownloadDocumentSaveDownloadPaths"

	//saveDownloadPaths is an array of arrays, which have these entries:
#define kURLIndex									0		//document url
#define kSaveDownloadPathIndex						1		//[self saveDownloadPath] value

	//if the document does not have a saveDownloadPath value (it's nil), it's not listed
	//in that array.
	

//private interface
@interface DownloadDocument (Private)
- __getSaveDownloadPath;
- __storeSaveDownloadPath;
@end


@implementation DownloadDocument

+ (void) initialize
{
	static bool foo = false;
	if (foo) return;
	foo = true;
	
	[[NSUserDefaults standardUserDefaults] registerDefaults:
		[NSDictionary dictionaryWithObject:[NSArray array]
									forKey:kUserDefault_downloadDoc_saveDownloadPaths]];
}

+ (NSArray*) recentTorrentDownloadFolders;
{
	id		result = nil;
	id		recentPaths, recentDocuments;
	id		ite, obj;
	id		fileManager;
	BOOL	isDir = NO;
	
	result = [NSMutableArray array];
	recentPaths = [[NSUserDefaults standardUserDefaults] objectForKey:kUserDefault_downloadDoc_saveDownloadPaths];
	recentDocuments = [[NSDocumentController sharedDocumentController] recentDocumentURLs];
	fileManager = [NSFileManager defaultManager];
	
	ite = [recentPaths objectEnumerator];
	while (obj = [ite nextObject])
	{
		id		path;
		id		url;
		
		url = [obj objectAtIndex:kURLIndex];
		if ([recentDocuments containsObject:[NSURL fileURLWithPath:url]])
		{
			path = [[obj objectAtIndex:kSaveDownloadPathIndex] stringByDeletingLastPathComponent];
			if ([result containsObject:path] == NO)
			{
				if ([fileManager fileExistsAtPath:path isDirectory:&isDir] && isDir)
				{
					[result addObject:path];	
				}
			}
		}
	}
	
	return result;
}

- (id)init
{
    self = [super init];
    if (self) 
	{
		__agent = [TomatoTorrentAgent new];
    }
    return self;
}

- (void) dealloc
{
	[__agent shutdown];
	[__agent release];
	[__metainfoDictionary release];
	[__metainfoPath release];
	[__saveToPath release];
	
	[super dealloc];
}

- (void)makeWindowControllers;
{
	__setupController = [DownloadSetupWindowController new];
		
	[self addWindowController:__setupController];
	[__setupController release];

	__progressController = [DownloadProgressController new];
	[self addWindowController:__progressController];
	[__progressController release];

	[__setupController showTorrentDownloadSetup];
	
	if (__metainfoDictionary != nil)
		[__setupController setTomatoMetainfoDictionary:__metainfoDictionary];
}

- (void)removeWindowController:(NSWindowController *)windowController;
{
	[super removeWindowController:windowController];
	
	if (windowController == __setupController)
		__setupController = nil;
	else if (windowController == __progressController)
		__progressController = nil;
}

//---------------------------------------------------------------------------
//		• - dataRepresentationOfType:
//---------------------------------------------------------------------------
//	Don't do no saving.

- (NSData *)dataRepresentationOfType:(NSString *)aType
{
    return nil;
}

//---------------------------------------------------------------------------
//		• - readFromFile:ofType:
//---------------------------------------------------------------------------
//	NSDocument loading.

- (BOOL)readFromFile:(NSString *)fileName ofType:(NSString *)type;
{
	if ([type isEqual:kTorrentDocumentType])
	{
		[__metainfoPath release];
		__metainfoPath = [fileName retain];
		
		//get the saved download path, if any, associated with this.
		[self __getSaveDownloadPath];
		
		//fetch info.
		[__agent metainfoPath:__metainfoPath delegate:self];
	}
	
	return YES;
}

- tomatoTorrentAgent
{
	return __agent;
}

- tomatoMetainfoDictionary
{
	return __metainfoDictionary;
}

//---------------------------------------------------------------------------
//		• - setSaveDownloadPath:
//---------------------------------------------------------------------------
//	both called by __getSaved and __storeSave, and the gui.

- (void) setSaveDownloadDirectoryPath:path;
{
	[__saveDownloadFolder release];
	__saveDownloadFolder = [path copy];
}

- saveDownloadDirectory
{
	return __saveDownloadFolder;
}

- (void) setStartsImmediately:(BOOL)foo;
{
	__startImmediately = foo;
}

- (BOOL) startsImmediately
{
	return __startImmediately;
}

- (void) setSaveDownloadPath:path
{
	[__saveToPath release];
	__saveToPath = [path retain];
	
	if (__didStart)
		[self __storeSaveDownloadPath];
}

- saveDownloadPath
{
	return [[__saveToPath retain] autorelease];
}

- (void) startDownload:sender
{
	if (__didStart ==  NO && __metainfoDictionary != nil)
	{
		__didStart = YES;
		[__setupController closeWithoutClosingDocument];
		
		if (__saveToPath == nil)
		{
			id	root, destName;
			
			root = [[SKInternetConfig instance] downloadFolderPath];
			if (root == nil) root = NSHomeDirectory();
			
			destName = [__metainfoDictionary objectForKey:kTomatoAgentKey_metainfo_arg_nameFs];
			//+++ not checking that....
			
			__saveToPath = [[root stringByAppendingPathComponent:destName] retain];
			
			NSLog(@"generated file path %@", __saveToPath);
		}
		
		[self __storeSaveDownloadPath];
		
		//get the agent to do its thing and show the tracker.
		[__agent startDownloadingToPath:__saveToPath delegate:self];
		
		[__progressController trackTorrentDownloadProgress];
	}
}

- (BOOL) didStartDownload
{
	return __didStart;
}

- tomatoTorrentAgent:agent metainfo:response;
{
	[__progressController currentActivityName];
	if ([response fault] == nil)
	{
		BOOL	fudge = NO;
		id		fileName;
		
		__metainfoDictionary = [[response responseContent] retain];
		fileName = [[__metainfoDictionary objectForKey:kTomatoAgentKey_metainfo_arg]
			objectForKey:kTomatoAgentKey_metainfo_arg_nameFs];
		
		if (__setupController)
			[__setupController setTomatoMetainfoDictionary:__metainfoDictionary];
		
		if (fileName
			&& __saveDownloadFolder
			&& [[NSFileManager defaultManager] fileExistsAtPath:__saveDownloadFolder isDirectory:&fudge]
			&& fudge == YES)
		{
			[self setSaveDownloadPath:[__saveDownloadFolder stringByAppendingPathComponent:fileName]];
		}

		if (__startImmediately) [self startDownload:nil];
	}
	
	return self;
}

- tomatoTorrentAgent:agent downloadStarted:response
{
	if (__inHaltResumeRequest && [response fault] == nil)
	{
		__inHaltResumeRequest = NO;
		__didHalt = NO;
		[__progressController trackTorrentDownloadProgress];
	}
	
	return self;
}

- tomatoTorrentAgent:agent downloadEnded:response
{
	if (__inHaltResumeRequest && [response fault] == nil)
	{
		__inHaltResumeRequest = NO;
		__didHalt = YES;
		[__progressController stopTrackingTorrentProgress];
	}
	
	return self;
}

- (void) stopTorrentDownload
{
	if (!__inHaltResumeRequest && __didStart)
	{
		__inHaltResumeRequest = YES;
		
		[__agent endDownload:self];
	}
}

- (void) restartTorrentDownload
{
	if (!__inHaltResumeRequest && __didStart)
	{
		__inHaltResumeRequest = YES;
		
		[__agent restartDownload:self];
	}
}

@end

@implementation DownloadDocument (MenuActions)
- (void) toggleTorrentRunningAction:sender
{
	if (__didHalt)
		[self restartTorrentDownload];
	else
		[self stopTorrentDownload];
}

- (BOOL) validateMenuItem:(NSMenuItem*)item
{
	SEL		action = [item action];
	BOOL	result = NO;
	
	if (action == @selector(toggleTorrentRunningAction:))
	{
		result = __didStart && !__inHaltResumeRequest;
		
		if (result)
		{
			if (__didHalt)
			{
				[item setTitle:NSLocalizedString(@"Start Torrent", @"start torrent menu item")];
			}
			else
			{
				[item setTitle:NSLocalizedString(@"Stop Torrent", @"stop torrent menu item")];
			}
		}
	}
	else
		result = [super validateMenuItem:item];
	
	return result;
}
@end

@implementation DownloadDocument (OSASupport)

- activity
{
	return [__progressController currentActivityName];
}

- statistics
{
	id	result = nil;
	
	result = [__progressController lastResponseContent];
	if (result == nil)
		result = [NSDictionary dictionary];
	
	return result;
}

- handleStartDownload:(NSScriptCommand *)command 
{
	id			args = [command evaluatedArguments];
    NSString 	*toPath = [args objectForKey:@"toPath"];
	NSString	*toFolder = [args objectForKey:@"toFolderPath"];

	if (toFolder && [toFolder length] > 0)
	{
		[self setSaveDownloadDirectoryPath:toFolder];
	}
	else if (toPath && [toPath length] > 0)
	{
		[self setSaveDownloadPath:toPath];
	}
	
	[self setStartsImmediately:YES];	
	[self startDownload:nil];
	
	return nil;
}

- handleDownloadStopCommand:(NSScriptCommand*)command
{
	[self stopTorrentDownload];
	return nil;
}

- handleDownloadRestartCommand:(NSScriptCommand*)command
{
	[self restartTorrentDownload];
	return nil;
}

@end

@implementation DownloadDocument (Private)
- (unsigned) __indexOfURLString:url inDefaultsArray:array
{
	id			object;
	unsigned	result 	= 0,
				count 	= [array count];
	BOOL		found	= NO;
	
	while (result < count)
	{
		object = [array objectAtIndex:result];
		
		if ([[object objectAtIndex:kURLIndex] isEqual:url])
		{
			found = YES;
			break;
		}
			
		++result;
	}
	
	if (!found) result = NSNotFound;
	
	return result;
}

- __getSaveDownloadPath;
{
	id		url, path, savedPaths;

	if (url = __metainfoPath)	//old cruft.
	{
		if (savedPaths = [[NSUserDefaults standardUserDefaults] arrayForKey:
								kUserDefault_downloadDoc_saveDownloadPaths])
		{
			//that's an array sorted by newest to oldest. we have to search
			//it for a matching URL. The array consists of (url, savePath) entries.
			unsigned	index = [self __indexOfURLString:url inDefaultsArray:savedPaths];
		
			path = nil;
			if (index != NSNotFound)
			{
				//if this path does not exist, we should use 'nil' to get the
				//default behaviour. This is supposed to be a path that the user
				//has already saved a file to, or a directory that they saved
				//the file in. Either way it is supposed to exist.
				//
				//Unfortunately, NSURL only saves Alias Records when 
				//NSDocumentController uses them :P
				
				path = [[savedPaths objectAtIndex:index] objectAtIndex:kSaveDownloadPathIndex];
				
				if ([[NSFileManager defaultManager] fileExistsAtPath:path] == NO)
					path = nil;
			}
			
			[self setSaveDownloadPath:path];
		}
	}
	
	return self;
}

- __storeSaveDownloadPath
{
	id			url, path, savedPaths;
	unsigned	index;

	if (url = __metainfoPath)	//old cruft.
	{
		//get existing array
		savedPaths = [[NSUserDefaults standardUserDefaults] arrayForKey:
								kUserDefault_downloadDoc_saveDownloadPaths];

		path = [self saveDownloadPath];
		index = [self __indexOfURLString:url inDefaultsArray:savedPaths];
		
		//make a mutable version
		savedPaths = [NSMutableArray arrayWithArray:savedPaths];

		if (index != NSNotFound)
			[savedPaths removeObjectAtIndex:index];
		
		if (path)
		{
			[savedPaths insertObject:[NSArray arrayWithObjects:url, path, nil]
							 atIndex:0];		//move to front, it's the most recent.
		}
		
		//limit it to 10 entries. It'd really be easier to store the save download
		//path in the resource fork of the .torrent, but it can't be done for non-file URLs anyway.
		if ([savedPaths count] > 10)
		{
			[savedPaths removeObjectsInRange:NSMakeRange(10, [savedPaths count] - 10)];
		}
		
		//store new array
		[[NSUserDefaults standardUserDefaults] setObject:savedPaths 
												  forKey:kUserDefault_downloadDoc_saveDownloadPaths];
	}
	
	return self;
}

@end
