//==============================================================================
//		• DownloadProgressController.m				TomatoTorrent
//==============================================================================
//  Created by Sarwat Khan on Tue Jun 03 2003.
//  Copyright (c) 2005, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khan’s name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//
//
//	There's too much logic in here to figure out what the agent is doing and how
//	to respond to it. I'd like much more of that logic to be in the Python script,
//	so this can just be a 'dumb' controller and just display whatever the python
//	script gives it.
//
//
//	Ideally, most of this code will be replaced by logic in the Python script and
//	intelligent formatter objects that will be obtained from... somewhere.
//
//
//		Doing a quick run over the source, it's not that bad actually. 
//		Most of the code in this file is,
//
//			- filling in edit fields in reloadData
//			- being the middle-man of TomatoTorrentAgent and the user
//			- managing timers for the progress bar/stats update interval so we
//			  don't hog the CPU (that pretty progress bar is CPU hungry)
//			- managing outline views and table views so the user can see the data
//
//		If the code got bigger, it would be best left to factoring out the logic here
//		to more specialized classes to provide a better, less cryptic user experience.


#import "DownloadProgressController.h"

#import "DownloadDocument.h"
#import "TomatoTorrentAgent.h"

#import "AppDelegate.h"
#import "PreferencesController.h"
#import "SKContainerFormatter.h"
#import "SKDataFormatters.h"

#import "ModuleOptionsController.h"	//+++ for formatterForKey. do something smarter.
#import "NSBox+DynamicContent.h"
#import "NSWindow+WindowContentSize.h"
#import "NSDictionary+DataTransform.h"

#import "TorrentFaultFormatter.h"
#import "TorrentConnectionRate.h"


//these two are for both tab views (main and details - the main content is a tab view too)
#define kStatsTabIdentifier				@"stats"
#define kErrorTabIdentifier				@"error"

//tag for title in main tab view's error tab.
#define	kErrorTitleViewTag				7			//7 is darker

//tags for configuration tab view's value tab view.
#define kConfigValueTabView_text		@"text"
#define kConfigValueTabView_bool		@"bool"

#define kDownloadingActivity			@"Downloading"
#define kDownloadingActivity_comment	@"downloading download activity"

#define kStartingActivity				@"Starting"
#define kStartingActivity_comment		@"starting download activity"

#define kCheckingActivity				@"checking existing download"
#define kCheckingActivity_comment		@"checking download activity"


#define kShutdownActivity				@"Shutdown"
#define kSeedingActivity				@"Seeding"
#define kLimboActivity					@"Limbo"


#define kNotAvailableString				@"N/A"

#define kKilobyte       1024.0
#define kMegabyte       (1024 * kKilobyte)
#define kGigabyte       (1024 * kMegabyte)

static id g_stats_table_thing = nil;

id	bt_format_file_size_object(id object);

	//download errors have a time too; not just strings.
@interface DownloadErrorFormatter: NSFormatter
{
	id		_timeFormatter;
}
@end

//I use this as the window's content view; it rounds the bottom corners of the window.
#define kCornerCurveRadius		10
@interface TorrentProgressWindowView: NSView
@end

@interface DownloadProgressController (Private)
- tomatoTorrentAgent:agent downloadState:response;
- tomatoTorrentAgent:agent downloadEnded:response;
- _setLastActivity:object;
- _localizedActivity;
- _setLastResponse:response;
- _showNoProgress;
- _showProgress;
- _postError:error;
- _displayDownloadFailure:response;
- _displayResponseFault:failureInfo;
- _removeTrackingTimer;
- _setFastTimer;
- _setSlowTimer;
- _noteReceivedConfigActionResponse;
- _noteWaitingForConfigActionResponse;
@end


//==============================================================================
//		• DownloadProgressController
//==============================================================================

@implementation DownloadProgressController

- init
{
	[super initWithWindowNibName:@"TorrentDownloadProgress"];

	_errorFormatter = [DownloadErrorFormatter new];

	_timerMethod = _slowTimer = @selector(_setSlowTimer);
	[self _setLastActivity:kLimboActivity];
	
	__validForAutostop = NO;	//until we figure out if we're downloading or not.
	
	if (g_stats_table_thing == nil)
	{
		g_stats_table_thing = [[NSArray alloc] initWithObjects:
			kTomatoAgentKey_downloadState_status_downRate,
			kTomatoAgentKey_downloadState_status_upRate2,
			kTomatoAgentKey_downloadState_status_downTotal,
			kTomatoAgentKey_downloadState_status_upTotal,
			kTomatoAgentKey_downloadState_status_numPeers,
			kTomatoAgentKey_downloadState_status_numSeeds,
			kTomatoAgentKey_downloadState_status_numCopies,
			//kTomatoAgentKey_downloadState_status_upRate2,
			//@"downtotal2",
			nil];
	}

	return self;
}


- (void) dealloc
{
	[_saveEmptyView release];
	[_lastResponse release];
	[__savedStats release];
	[__lastTorrentConnectionRate release];
	[_lastActivity release];
	[__seedingStartDate release];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	[super dealloc];
}

//------------------------------------------------------------------------------
//		• windowDidLoad
//------------------------------------------------------------------------------
//	sets default formatters, shows the right tab view, other initial stuff.

- (void) windowDidLoad
{
	id			naString = SKLocalizedString(kNotAvailableString, nil);
	NSWindow	*window = [self window];
	
	//swap out the original content view for a nicer one.
	{
		id		content;
		id		oldContent;
		
		content = [[TorrentProgressWindowView alloc] initWithFrame:[[window contentView] bounds]];
		oldContent = [[window contentView] retain];
		[window setContentView:content];
		[oldContent setFrame:[content bounds]];
		[content addSubview:oldContent];
		[oldContent release];
		[content release];
		
		[oldContent setAutoresizingMask:(NSViewWidthSizable | NSViewHeightSizable)];
		[window setOpaque:NO];		
	}
	
	//sometimes when playing with the nib I end up setting the error. It's annoying.
	[_mainTabView  selectTabViewItemWithIdentifier:kStatsTabIdentifier];
	[_detailsTabView selectTabViewItemWithIdentifier:kStatsTabIdentifier];
	
	[_fileNameField setStringValue:SKLocalizedString(@"Starting...", @"download window filename startup")];
	[_downloadRateField setStringValue:naString];
	[_lastErrorField setStringValue:naString];
	[_timeLeftField setStringValue:@""];
	[_uploadRateField setStringValue:naString];
	[_timeLeftField setStringValue:naString];

	//initialize download rate.
	[_rateText setStringValue:@""];
	[_rateText setFormatter:[[TorrentConnectionRateFormatter new] autorelease]];
	
	[self _showNoProgress];
	
	//save initial window size
	//the advanced options is assumed to be hidden.
	_saveEmptyView = [[_detailsBox contentView] retain];
	[[self window] setShowsResizeIndicator:NO];
	
	
	{
		id		formatter;
		
		//set formatter for table cells that display containers (array, dictionary)
			//+++ is this still used? app delegate does the formatting now based on key.
		formatter = [[SKContainerFormatter new] autorelease];
		[[[_statsTable tableColumnWithIdentifier:@"value"] dataCell] setFormatter:formatter];
	
		//localized strings for blah.
		formatter = [[[[SKLocalizedStringFormatter new] autorelease] 
						setTableName:kTomatoAgentStringsTable_download] setBundle:TTAgentBundle_()];
	}
	
	
	[self _setLastActivity:kStartingActivity];
	
	//window content size
	{
		NSRect	frame, 
				emptyFrame;
		
		frame = [[[self window] contentView] frame];
		_smallContentSize = frame.size;
		[[self window] setMinimumContentSize:frame.size maximumSize:frame.size];
		
		frame = [_detailsView frame];
		emptyFrame = [_saveEmptyView frame];
		_expandedContentSize.width = _smallContentSize.width;
		_expandedContentSize.height = _smallContentSize.height + frame.size.height - emptyFrame.size.height;
	}
	
	//smaller font for error text view,
	[_errorText setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	
	//initialize timers and listen to notifs for when we need to switch between the two
	_timerMethod = @selector(_setSlowTimer);
	if ([NSApp isActive])
		_timerMethod = @selector(_setFastTimer);
		
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(_applicationActiveStatus:)
												 name:NSApplicationDidBecomeActiveNotification
											   object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(_applicationActiveStatus:)
												 name:NSApplicationDidResignActiveNotification
											   object:nil];
	
	//listen to prefs changing about autostop
	[[NSNotificationCenter defaultCenter] addObserver: self
											 selector: @selector(__autoStopPrefsChangedNotification:)
												 name: kTomatoAutoStopPrefsDidChangeNotification
											   object: nil];
}

- (void)synchronizeWindowTitleWithDocumentName;
{
	//no.	see reloadData for an explanation.
}

//---------------------------------------------------------------------------
//		• - currentActivityName
//---------------------------------------------------------------------------
//	This is just for applescript.
- currentActivityName
{
	/* I've wound up with an intertwined mess. I don't like the string value of _lastActivity */
	static id currentActivityMap = nil;
	id		result;
	
	if (currentActivityMap == nil)
	{
		currentActivityMap = [[NSDictionary alloc] initWithObjectsAndKeys:
			@"downloading",
			kDownloadingActivity,
			
			@"starting",
			kStartingActivity,
			
			@"checking existing download",
			kCheckingActivity,
			
			@"shutdown",
			kShutdownActivity,
			
			@"seeding",
			kSeedingActivity,
			
			@"limbo",
			kLimboActivity,
			
			nil];
	}
	
	result = [currentActivityMap objectForKey:_lastActivity];
	if (result == nil) result = @"limbo";
	
	return result;
}

//------------------------------------------------------------------------------
//		• trackTorrentDownloadProgress
//------------------------------------------------------------------------------
//	Activation. Download must have been started (will call state on it; calling
//	state when no dawnload is a fault).

- trackTorrentDownloadProgress
{
	_isTrackingProgress = YES;
	//_isFinished = NO;	//now this can be called multiple times (you can stop/start torrents)
	
	[self setShouldCascadeWindows:YES];
	
	[self reloadData];
	[[self window] makeKeyAndOrderFront:nil];
	
	[self performSelector:_timerMethod];
	
	if ([[[self document] tomatoTorrentAgent] downloadConfigurationForKey:kTomatoAgentKey_downloadParameter_maxUploadRate delegate:self])
		[self _noteWaitingForConfigActionResponse];	//else meh. odd though.
	
	_progressTickleTimer = [NSTimer scheduledTimerWithTimeInterval: 0.3 
															target: self 
														  selector: @selector(_tickleProgress:) 
														  userInfo: nil
														   repeats: YES];
	
	return self;
}

//------------------------------------------------------------------------------
//		• stopTrackingTorrentProgress
//------------------------------------------------------------------------------
//	Mostly used when a failure is detected.

- stopTrackingTorrentProgress
{
	[self _removeTrackingTimer];
	[_progressTickleTimer invalidate], _progressTickleTimer = nil;
	
	if (_isTrackingProgress)
	{
		_isTrackingProgress = NO;
		
		[self _showNoProgress];
		
		[self _setLastResponse:nil];
		[_activityText setStringValue:NSLocalizedString(@"Stopped", nil)];
		[_rateText setObjectValue:@""];
		if (!_isFinished)	//otherwise "Download Finished" is acceptable.
			[[self window] setTitle:NSLocalizedString(@"Stopped", nil)];
	}
	return self;
}

//------------------------------------------------------------------------------
//		• failedResponse:answeredTo:
//------------------------------------------------------------------------------
//	GUI for showing failed responses by the model (document).
//
//	response must have a 'failure' key.

- failedResponse:response answeredTo:(SEL)selector;
{
	SKLog_(@"failed response.");
	[self _setLastResponse:response];
	[self stopTrackingTorrentProgress];	//reload data will cause this anyway
	[self reloadData];
	
	return self;
}

- (BOOL) shouldCloseDocument
{
	return YES;
}

- (void) windowWillClose:notif
{
	[self _removeTrackingTimer];
	if (_isTrackingProgress)
		[[[self document] tomatoTorrentAgent] endDownload:nil];
	[self stopTrackingTorrentProgress];
}

//------------------------------------------------------------------------------
//		• torrentDownloadDidStartup:
//------------------------------------------------------------------------------
//	Sent by document once the startup response comes in.
//
//	Nothing to do here anymore. The response data after the download start is empty.

- torrentDownloadDidStartup:response;
{
	return self;
}

- (void) showWindow:sender
{
	if (_isTrackingProgress) [super showWindow:sender];
}

//------------------------------------------------------------------------------
//		• reloadData
//------------------------------------------------------------------------------
//	this thing got bigger and bigger and bigger...
//
//	The logic is also less than ideal because it makes up for some functionality
//	that the agent script didn't originally have. Then it got it.
//
//	Then BitTorrent 4 was hacked onto it.
//
//	It's a bit of a mess.

- (void) reloadData
{
	id		object;
	id		data = [_lastResponse responseContent];
	id		errors = nil;
	
	if (_lastResponse)
	{
		double		percentDone = 0;
		id			fractionDone = nil;
		id			name = nil;
		id			display;	//this variable is reused a LOT.
		id			downloadStatus;
		id			activity;
		id			size = nil;
		BOOL		isFinished = NO;
		BOOL		didFail = NO;
		
		id			tomatoInfo = [[[self document] tomatoMetainfoDictionary] objectForKey:kTomatoAgentKey_metainfo_arg];
		
		downloadStatus = [data objectForKey:kTomatoAgentKey_downloadState_status];
		
		//--- activity
		if (activity = [downloadStatus objectForKey:kTomatoAgentKey_downloadState_status_activity])
		{
			//actually downloading a torrent, so put the percentage done in the title bar
			if ([activity isEqual:kTomatoAgentKey_downloadState_status_activity_downloading])
			{
				__validForAutostop = YES;	//if we were ever downloading, then we can respond to auto-stop.
				_isFinished = NO;
				[self _setLastActivity:kDownloadingActivity];
			}
			else if ([activity isEqual:kTomatoAgentKey_downloadState_status_activity_seeding])
			{
				_isFinished = isFinished = YES;
				[self _setLastActivity:kSeedingActivity];
				
				if (__seedingStartDate == nil)
				{
					__seedingStartDate = [NSDate new];
					
					//valid for auto stop: this is set to true if we ever downloaded. We only use the secondary
					//upload rate if we are valid for auto stop. validForAutostop is set whether or not autostop is on.
					if (__validForAutostop && [PreferencesController usesSecondaryUploadRateForCompletedTorrents])
					{
						if ([[[self document] tomatoTorrentAgent] configureDownloadKey: kTomatoAgentKey_downloadParameter_maxUploadRate 
																			 withValue: [NSNumber numberWithInt:[PreferencesController secondaryUploadRateKBs]]
																			  delegate: self] != nil)
						{
							[self _noteWaitingForConfigActionResponse];
						}
					}
				}
			}
			else if ([activity isEqual:kTomatoAgentKey_downloadState_status_activity_checkingFile])
			{
				[self _setLastActivity:kCheckingActivity];
			}
			else if ([activity isEqual:kTomatoAgentKey_downloadState_status_activity_shutDown])
			{
				[self stopTrackingTorrentProgress];
				[self _setLastActivity:kShutdownActivity];
			}
			else if ([activity isEqual:kTomatoAgentKey_downloadState_status_activity_startup])
			{
			}
			else
			{
				didFail = YES;
				[self stopTrackingTorrentProgress];
			}
			
			[_activityText setStringValue:NSLocalizedString(activity, nil)];
		}
			//more status handling below, 'status again'
		
		//--- fraction done
		if (fractionDone = [downloadStatus objectForKey:kTomatoAgentKey_downloadState_status_fractionDone])
		{
			if (isFinished == NO)
				percentDone = ([fractionDone doubleValue] * 100);
			else
				percentDone = 100;	//also done below in status again... I should determine which one actually happens.
		}

		//--- progress bar; should have a meaningful activity set.
		if (fractionDone && [fractionDone respondsToSelector:@selector(doubleValue)])
		{
			if (percentDone != _lastPercentDone)
			{
				[_progressBar setDoubleValue:[fractionDone doubleValue]];
				_lastPercentDone = percentDone;
				[self _showProgress];
			}
		}
		
		//status again: adjust the progress bar in relation to certain status.
		if (isFinished)
		{
			if (fractionDone == nil && _isTrackingProgress)
			{
				if ([_progressBar doubleValue] != 1) 
					[_progressBar setDoubleValue:1];
				[_progressBar stopAnimation:nil];
				[_progressBar setIndeterminate:NO];
			}
		}

		//--- file name
		//	
		//	name is not in the window title for a few reasons. The first is that the name
		//	is often long, and the window width is often short. Secondly, BT may change
		//	the file name, so setRepresentedFilename: may not be valid. Plus, the file
		//	may be a directory (although that may be acceptable anyway). The last two
		//	points aren't big deals (they can be worked around), but when combined with the
		//	first point, I'd rather write this chunky comment.
		
		
		//if (object = [data objectForKey:kTomatoAgentKey_downloadState_name])
		{
			name = [tomatoInfo objectForKey:kTomatoAgentKey_metainfo_arg_nameFs];
			display = [NSString stringWithFormat:@"%@", name];
			
			[_fileNameField setStringValue:display];
		}
		if (_isTrackingProgress == NO || [_lastResponse fault] != nil)
		{
			display = [[[self document] tomatoMetainfoDictionary] objectForKey:kTomatoAgentKey_metainfo_arg_nameFs];
			//if (display == nil) display = [[[self document] tomatoTorrentAgent] sourceURL];
			if (display == nil) display = @"";
			
			[_fileNameField setStringValue:display];
		}
		
		//--- download/upload rate
		{
			display = [[TorrentConnectionRate new] autorelease];
			[display takeValuesFromDictionary:downloadStatus];
			[_rateText setObjectValue:display];
			
			if (__lastTorrentConnectionRate)
				[__lastTorrentConnectionRate release];
			__lastTorrentConnectionRate = [display retain];
		}
		
		//--- time left/size
		size = [tomatoInfo objectForKey:kTomatoAgentKey_metainfo_arg_totalBytes];
		if (!isFinished && [activity isEqual:kTomatoAgentKey_downloadState_status_activity_downloading])
			size = [NSString stringWithFormat:@"%0.1f%% of %@", [fractionDone doubleValue]*100, bt_format_file_size_object(size)];
		else
			size = [NSString stringWithFormat:@"%@", bt_format_file_size_object(size)];
		
		if (object = [downloadStatus objectForKey:kTomatoAgentKey_downloadState_status_hmsTimeEstimate])
		{
			display = [[NSApp delegate] stringForObjectValue:object forKey:kTomatoAgentKey_downloadState_status_hmsTimeEstimate];
			
			[_timeLeftField setObjectValue:[NSString stringWithFormat:SKLocalizedString(@"%@ * %@", nil), display, size]];
		}
		else
			[_timeLeftField setStringValue:size];

		//window title
		if (activity)
		{
			//actually downloading a torrent; percentage is in the file name field.
			if ([activity isEqual:kTomatoAgentKey_downloadState_status_activity_downloading])
				display = SKLocalizedString(@"Downloading Torrent", @"Download Window Title - Downloading");
			else if ([activity isEqual:kTomatoAgentKey_downloadState_status_activity_seeding])
				display = SKLocalizedString(@"Download Finished", @"");
			else if ([activity isEqual:kTomatoAgentKey_downloadState_status_activity_checkingFile])
				display = SKLocalizedString(@"checking existing download", @"");
			
			//not downloading a torrent but something's happening. put the activity and percentage in title bar
			else if (fractionDone)
			{
				display = SKLocalizedString(@"%@ (%.1f%%)", @"Download Window Title (activity, percentDone)");
				display = [NSString stringWithFormat:display, [self _localizedActivity], percentDone];
			}
			
			//not downloading and no progress, put the status in the title bar. Might be 'failed'
			else
			{
				display = [NSString stringWithFormat:SKLocalizedString(@"Download - %@", 
					@"Download - (status OR name) - window OR miniwindow title"),
					TTAgentLocalizedString_(activity, kTomatoAgentStringsTable_downloadState)];
			}
			
			if ([[[self window] title] isEqual:display] == NO)
			{
				//This is also a good oppurtunity to remove the error display.
				[_mainTabView selectTabViewItemWithIdentifier:kStatsTabIdentifier];
				[[self window] setTitle:display];
			}
			
			//put the progress in the mini window title. It would be cool if the mini-window image is a progress bar.
			if (name != nil && fractionDone != nil)
			{
				display = SKLocalizedString(@"%@: %@ (%.1f%%)", @"Mini Window Download (activity, name, percentDone)");
				display = [NSString stringWithFormat:display, 
					TTAgentLocalizedString_(activity, kTomatoAgentStringsTable_downloadState), name, percentDone];
				[[self window] setMiniwindowTitle:display];
			}
			else if (name)
			{
				display = [NSString stringWithFormat:
					SKLocalizedString(@"Download - %@", 
						@"Download - (status OR name) - window OR miniwindow title"),
					name];
				[[self window] setMiniwindowTitle:display];
			}
			
			//if (object = [data objectForKey:kTomatoAgentKey_downloadState_destPath])
			object = [[self document] saveDownloadPath];
			{
				if ([[[self window] representedFilename] isEqual:object] == NO)
				{
					//NSLog(@"set represented: %@", [object description]);
					[[self window] setRepresentedFilename:object];
				}
			}
		}

		//outline view
		if (_freezeStatsTable == NO)
		{
			//update copy of displayed data
			[__savedStats release];
			__savedStats = [[NSMutableDictionary alloc] initWithDictionary:downloadStatus];
			if ([data objectForKey:kTomatoAgentKey_downloadState_downtotal])
				[__savedStats setObject:[data objectForKey:kTomatoAgentKey_downloadState_downtotal]
								 forKey:@"downtotal2"];

			//and reload now.
			[_statsTable reloadData];
		}
		
		//errors. It returns an array of all the errors.
		errors = [data objectForKey:kTomatoAgentKey_downloadState_errors];
		if (errors && [errors count] != _lastErrorCount)
		{
			id		currentError = [errors lastObject];
			NSRange	thing;
			
			_lastErrorCount = [errors count];
			
			//don't post stupid fast-resume errors.
			thing = [[_errorFormatter stringForObjectValue:currentError] rangeOfString:@"fastresume"];
			if (thing.location == NSNotFound)
				[self _postError:currentError];
			
			//update the text view.
			{
				id			text = [NSMutableString string];
				id			ite, entry;
				
				{
					ite = [errors reverseObjectEnumerator];
					while (entry = [ite nextObject])
					{
						[text appendString:[_errorFormatter stringForObjectValue:entry]];
						[text appendString:@"\n\n"];
					}
					
					[_errorText setString:text];
					
				}
			}
		}
		
		//check faults.
		if (errors = [_lastResponse fault])
		{
			if (didFail)
				[self _displayDownloadFailure:errors];
			else
				[self _displayResponseFault:errors];
		}
		
		[self stopTorrentIfNowIsAGoodTimeToDoThat];
	}	
}

//------------------------------------------------------------------------------
//		• toggleDetails:
//------------------------------------------------------------------------------

- (void) toggleDetails:sender
{
	id			newView 		= _saveEmptyView;
	BOOL		showingDetails 	= [sender state] == NSOnState;
	NSSize		minSize, maxSize;
	
	[[self window] setShowsResizeIndicator:showingDetails];

	if (showingDetails) newView = _detailsView;

	[_detailsBox switchContentViewTo:newView showingViewDuringResize:_saveEmptyView];
	
	if (showingDetails) 
	{
		[self reloadData];
		minSize = _expandedContentSize;
		maxSize = NSMakeSize(800, 0);
	}
	else
		minSize = maxSize = _smallContentSize;
		
	[[self window] setMinimumContentSize:minSize maximumSize:maxSize];
	
	//draw now.
	[_detailsBox display];
}


- (BOOL)windowShouldZoom:(NSWindow *)window toFrame:(NSRect)newFrame;
{
	return [_detailsBox contentView] == _detailsView;
}

- __nag
{
	NSBeginAlertSheet(SKLocalizedString(
										@"Can't Configure BitTorrent", nil), 	//title; is this displayed for sheets?
					  nil,									//default button
					  nil,									//alt button
					  nil,									//other button,
					  [self window],
					  nil,									//modal delegate
					  nil,									//didEndSelector
					  nil,									//didDismiss
					  NULL,									//context,
					  SKLocalizedString(@"BitTorrent is currently too busy to be configured. Try again when the progress bar is not striped.", nil));
	return self;
}

- (void) configValuesEdited:sender
{
	id		value = [sender objectValue];
	
	if (value == nil) value = [NSNumber numberWithInt:0];
	
	if ([[[self document] tomatoTorrentAgent] configureDownloadKey: kTomatoAgentKey_downloadParameter_maxUploadRate 
														 withValue: value 
														  delegate: self] != nil)
	{
		[self _noteWaitingForConfigActionResponse];
	}
	else
	{
		[self __nag];
	}
}

- (void) changePort:sender
{
	if ([[[self document] tomatoTorrentAgent] configureDownloadKey: kTomatoAgentKey_downloadParameter_changePortAction 
														 withValue: @"foo" 
														  delegate: self] == nil)
	{
		[self __nag];
	}
}

//------------------------------------------------------------------------------
//		• freezeStatsTable:
//------------------------------------------------------------------------------

- (void) freezeStatsTable:sender
{
	if (sender == _statsFreezeButton)
		_freezeStatsTable = [sender intValue];
	else
	{
		_freezeStatsTable = YES;
		[_statsFreezeButton setState:NSOnState];
	}
}

- lastResponseContent
{
	return [_lastResponse responseContent];
}

- (void) stopTorrentIfNowIsAGoodTimeToDoThat
{
	if (_isFinished && __validForAutostop && [PreferencesController doAutoStopTomatoTorrentDownloads])
	{
		//check the auto-stop type and stop if applicable.
		switch ([PreferencesController tomatoDownloadAutoStopMethod])
		{
			case kAutoStopTorrentMethod_shareRatio:
				if (__lastTorrentConnectionRate)
				{
					int		up, down;
					float	ratio;
					
					up = [[__lastTorrentConnectionRate valueForKey:@"upTotal"] intValue];
					down = [[__lastTorrentConnectionRate valueForKey:@"downTotal"] intValue];
					
					if (up >= 0 && down > 0)
					{
						ratio = up;
						ratio /= down;
						
						if (ratio >= [PreferencesController tomatoTorrentTargetDownloadShareRatio])
							[[self document] stopTorrentDownload];
					}
				}
			break;
				
			case kAutoStopTorrentMethod_dataLimit:
				if (__lastTorrentConnectionRate)
				{
					unsigned		upTotal;
					
					upTotal = [[__lastTorrentConnectionRate valueForKey:@"upTotal"] intValue];
					upTotal /= 1024 * 1024;	//megabytes.
					
					if (upTotal >= [PreferencesController seedingUploadLimitAsMegabytes])
						[[self document] stopTorrentDownload];
				}
			break;
				
			case kAutoStopTorrentMethod_timeLimit:
				if (__seedingStartDate)
				{
					if (([NSDate timeIntervalSinceReferenceDate] - [__seedingStartDate timeIntervalSinceReferenceDate])
						>= [PreferencesController seedingAutoStopTimeInterval])
						[[self document] stopTorrentDownload];
				}
			break;
		}
	}
}
@end

@implementation DownloadProgressController (Private)

//------- TomatoTorrentAgent delegate methods

- tomatoTorrentAgent:agent downloadEnded:response;
{
	[self _setLastResponse:response];
	[self stopTrackingTorrentProgress];
	[self reloadData];	//shows the progress bar's value.
	return self;
}

- tomatoTorrentAgent:agent downloadState:response
{
	if (_isTrackingProgress)
	{
		[self _setLastResponse:response];
		[self reloadData];
	}
	
	return self;
}

- __blargarb:response
{
	id		uprate = [[response responseContent] objectForKey:kTomatoAgentKey_downloadParameter_maxUploadRate];
	
	if (uprate)
	{
		[__uprateEditText setStringValue:uprate];
	}
	[self _noteReceivedConfigActionResponse];
	return self;
}

- tomatoTorrentAgent:agent downloadConfiguration:response
{
	[self __blargarb:response];
	
	return self;
}

- tomatoTorrentAgent:agent configuredDownload:response
{
	[self __blargarb:response];
	return self;
}

- tomatoTorrentAgent:agent backloggedRequest:requestName wasRejected:(BOOL)rejected backlogCount:(int)effectiveCount;	
{
	//egads, we're backlogged? put up a no progress sign.
	[self _showNoProgress];
	return self;
}

- (int)numberOfRowsInTableView:(NSTableView *)tableView;
{
	return [g_stats_table_thing count];
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row;
{
	if ([[tableColumn identifier] isEqual:@"key"])
		return NSLocalizedString([g_stats_table_thing objectAtIndex:row], nil);
	else
		return [[NSApp delegate] stringForObjectValue:[__savedStats objectForKey:[g_stats_table_thing objectAtIndex:row]]
											   forKey:[g_stats_table_thing objectAtIndex:row]];
}

- (BOOL)tableView:(NSTableView *)tableView shouldSelectRow:(int)row;
{
	return NO;
}

- _setLastActivity:object;
{
	//if ([object isEqual:_lastActivity] == NO)
	//	SKLog_(@"set activity: %@", object);

	[_lastActivity release];
	_lastActivity = [object retain];
	return self;
}

- _localizedActivity
{
	id			result = @"";
	static id 	selfDefined = nil;
	
	if (0)
	{
		//this is just so genstrings sees them in use and generates strings
		result = SKLocalizedString(kDownloadingActivity, kDownloadingActivity_comment);
		result = SKLocalizedString(kStartingActivity, kStartingActivity_comment);
		result = SKLocalizedString(kCheckingActivity, kCheckingActivity_comment);
	}
	
	if (selfDefined == nil)
	{
		selfDefined = [NSSet setWithObjects:kDownloadingActivity, 
											kStartingActivity,
											kShutdownActivity,
											kCheckingActivity,
											kSeedingActivity,
											nil];
		[selfDefined retain];
	}
	
	if (_lastActivity)
	{
		if ([selfDefined containsObject:_lastActivity])
			result = SKLocalizedString(_lastActivity, nil);
		else
			result = TTAgentLocalizedString_(_lastActivity, kTomatoAgentStringsTable_downloadState);
	}
	
	return result;
}

- _setLastResponse:response;
{
	[_lastResponse release];
	_lastResponse = [response copy];
	return self;
}

//------------------------------------------------------------------------------
//		• _showNoProgress/_showProgress/_tickleProgress:
//------------------------------------------------------------------------------
//	The progress bar sucks a lot of CPU time (like 5-10% of the CPU for each progress
//	bar) so I only use 'startAnimation' while the app is the foreground app.
//	Otherwise, the tickeProgress: timed method will animate it every 1/3 of a second.

- _showNoProgress;
{
	_showingProgress = NO;
	if (_isTrackingProgress)
	{
		_animatingProgress = YES;
		if ([NSApp isActive])
			[_progressBar startAnimation:nil];		//else the tickle timer will do it.
		[_progressBar setIndeterminate:YES];
	}
	else
	{
		[_progressBar stopAnimation:nil];
		_animatingProgress = NO;
	}

	return self;
}

- _showProgress;
{
	if (_isTrackingProgress)
	{
		id		status = [[_lastResponse responseContent] objectForKey:kTomatoAgentKey_downloadState_status];
		
		[_progressBar setIndeterminate:NO];

		[_mainTabView selectTabViewItemWithIdentifier:kStatsTabIdentifier];

		//only start animation if status is connecting or downloading
		//	
		if (status && 
			([status isEqual:kTomatoAgentKey_downloadState_status_activity_downloading]))
		{
			if ([NSApp isActive])
				[_progressBar startAnimation:nil];
			
			_animatingProgress = YES;
		}
		
		_showingProgress = YES;
		
		if ([NSApp isActive] == NO)
			[_progressBar stopAnimation:nil];	//setting the value of a progress bar will start animation.
												//whenever I set the value I call this method, so this will
												//stop it if needed.
	}
	else
		[_progressBar stopAnimation:nil],
		_animatingProgress = NO;

	return self;
}

- (void) _tickleProgress:timer
{
	if (_animatingProgress && [NSApp isActive] == NO) [_progressBar animate:nil];
}

- _postError:error
{
	id		object;
	
	if (!_isFinished)	//who really cares if we're finished.
	{
		[_mainTabView selectTabViewItemWithIdentifier:kErrorTabIdentifier];
		[self _showNoProgress];
	}

	//set string value
	object = [_errorFormatter stringForObjectValue:error];
	[_lastErrorField setStringValue:object];
	
	return self;
}


//------------------------------------------------------------------------------
//		• _displayDownloadFailure:
//------------------------------------------------------------------------------
//	User likely provided unacceptable data. Unfortunately, bad torrent files
//	do not generate exceptions, so that has to be handled seprately (which is
//	by showing the 'errors')

- _displayDownloadFailure:info;
{
	id		errorTitle;
	id		formatter;
	
	[self stopTrackingTorrentProgress];

	[_mainTabView selectTabViewItemWithIdentifier:kErrorTabIdentifier];

	errorTitle = [_mainTabView viewWithTag:kErrorTitleViewTag];
	[errorTitle setStringValue:SKLocalizedString(@"Not Running", @"download failure title")];
	
	formatter = [[TorrentFaultFormatter new] autorelease];
	[formatter setDownloadFailureResponse:_lastResponse fromAgent:[[self document] tomatoTorrentAgent]];

	[_lastErrorField setObjectValue:info];
	[_lastErrorField setFormatter:formatter];
	
	return self;
}

//------------------------------------------------------------------------------
//		• _displayResponseFault:
//------------------------------------------------------------------------------
//	Called from reloadData.
//
//	This is the gui for showing any failure response. This is typically that the
//	download cannot be started because bad data was given (connection and other
//	 network errors are handled gracefully by BitTorrent, and show up as 'errors').

- _displayResponseFault:failureInfo;
{
	id		errorTitle;
	id		formatter;
	
	[self stopTrackingTorrentProgress];

	[_mainTabView selectTabViewItemWithIdentifier:kErrorTabIdentifier];

	errorTitle = [_mainTabView viewWithTag:kErrorTitleViewTag];
	[errorTitle setStringValue:SKLocalizedString(@"Not Running Due To Fault:", @"post response fault title")];
	
	formatter = [[TorrentFaultFormatter new] autorelease];

	[_lastErrorField setObjectValue:failureInfo];
	[_lastErrorField setFormatter:formatter];
	
	return self;
}

- (void) _progressUpdateTimer:timer
{
	if ([[[self document] tomatoTorrentAgent] downloadState:self] == nil)
	{
		[self _showNoProgress];
	}
}

- _removeTrackingTimer
{
	if (_timer)
	{
		[_timer invalidate];
		[_timer release];
		_timer = nil;
		return self;
	}
	return nil;
}

- _setSlowTimer
{
	[self _removeTrackingTimer];
	if (_isTrackingProgress)
	{
		_timer = [NSTimer scheduledTimerWithTimeInterval:5.0
												  target:self 
												selector:@selector(_progressUpdateTimer:) 
												userInfo:nil 
												 repeats:YES];
		[_timer retain];
	}
	
	return self;
}

- _setFastTimer
{
	[self _removeTrackingTimer];
	if (_isTrackingProgress)
	{
		_timer = [NSTimer scheduledTimerWithTimeInterval:1.0
												  target:self 
												selector:@selector(_progressUpdateTimer:) 
												userInfo:nil 
												 repeats:YES];
		[_timer retain];
	}
	
	return self;
}

- _noteWaitingForConfigActionResponse
{
	++_configWaitNum;
	[__uprateEditText setEnabled:NO];
	[__uprateConfigProgress startAnimation:nil];
	return self;
}

- _noteReceivedConfigActionResponse
{
	--_configWaitNum;
	
	if (_configWaitNum <= 0)
	{
		_configWaitNum = 0;		//meh.
		[__uprateConfigProgress stopAnimation:nil];
		[__uprateEditText setEnabled:YES];
	}
	
	return self;
}

- (void) _applicationActiveStatus:notif
{
	_timerMethod = @selector(_setSlowTimer);
	
	if ([NSApp isActive])
	{
		_timerMethod = @selector(_setFastTimer);
		if (_animatingProgress) [_progressBar startAnimation:nil];
	}
	else if (_animatingProgress)
		[_progressBar stopAnimation:nil];			//that bugger eats a lot of CPU time.
													//the tickle timer will take over from here.
		
	if (_isTrackingProgress)
		[self performSelector:_timerMethod];
}

- (void) __autoStopPrefsChangedNotification:notif
{
	[self stopTorrentIfNowIsAGoodTimeToDoThat];
}

@end

@implementation DownloadProgressController (NSTabViewDelegate)
- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem;
{
	if (tabView == _detailsTabView) [self reloadData];
}
@end

//==============================================================================
//		• DownloadErrorFormatter
//==============================================================================

@implementation DownloadErrorFormatter
- init
{
	_timeFormatter = [SKUnixTimeFormatter new];

	return [super init];
}

- (void) dealloc
{
	[_timeFormatter release];
	[super dealloc];
}

- (NSString *)stringForObjectValue:(id)obj;
{
	id		result = nil;
	
	if ([obj respondsToSelector:@selector(count)] && [obj count] > 1)	
	{
		id			desc, seconds, date;
		
		desc = [obj objectAtIndex:1];
		seconds = [obj objectAtIndex:0];
		date = [_timeFormatter stringForObjectValue:seconds];
		
		result = [NSString stringWithFormat:
			NSLocalizedString(@"%@, (at %@)", @"error format (exception desc, date-time)"),
			desc, date]; 
	}
	else
		result = [obj description];
	
	return result;
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error;
{
	return NO;
}
@end


//---------------------------------------------------------------------------
//		• @implementation TorrentProgressWindowView
//---------------------------------------------------------------------------
//	fuggy.

@implementation TorrentProgressWindowView
- (void) drawRect:(NSRect)update
{
	NSRect		bounds;
	
	bounds = [self bounds];
	
	[[NSColor clearColor] set];
	NSRectFill(bounds);
	
	[[NSColor windowBackgroundColor] set];
	
	if (![[self window] showsResizeIndicator])
	{
		//draw an obloid.
		NSBezierPath	*path;
		
		path = [NSBezierPath new];
		
		[path moveToPoint:NSMakePoint(0, bounds.size.height)];
		[path lineToPoint:NSMakePoint(0, kCornerCurveRadius)];
		[path curveToPoint: NSMakePoint(kCornerCurveRadius, 0)
			 controlPoint1: NSMakePoint(0, 0)
			 controlPoint2: NSMakePoint(0, 0)];
		[path lineToPoint:NSMakePoint(bounds.size.width - kCornerCurveRadius, 0)];
		[path curveToPoint: NSMakePoint(bounds.size.width, kCornerCurveRadius)
			 controlPoint1: NSMakePoint(bounds.size.width, 0)
			 controlPoint2: NSMakePoint(bounds.size.width, 0)];
		[path lineToPoint:NSMakePoint(bounds.size.width, bounds.size.height)];
		[path closePath];
		[path fill];
		[path release];
	}
	else
	{
		[NSBezierPath fillRect:[self bounds]];
	}
}
@end


//==============================================================================
//		• stuff
//==============================================================================

id	
bt_format_file_size_object(id object)
{
	return [[NSApp delegate] formatDataSize:object];
}

