//==============================================================================
//		• DownloadSetupWindowController.m
//==============================================================================
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khan’s name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "DownloadSetupWindowController.h"

#import "DownloadDocument.h"
#import "TomatoTorrentAgent.h"

#import "PreferencesController.h"
#import "ModuleOptionsController.h"
#import "SKInternetConfig.h"
#import "TorrentFaultFormatter.h"
#import "AppDelegate.h"
#import "SKRange.h"
#import "NSBox+DynamicContent.h"
#import "SKContainerFormatter.h"
#import "NSFileManager+LocalizedPaths.h"

	//for the outline view.
#define kRootKey		@"/"

#define kItemKey		0
#define kItemValue		1
#define kItemParentPathKey	2


@interface DownloadSetupWindowController (Private)
- __agent;
- __setAutoStartTimer:timer;
- __setAutoStart:(BOOL)start;
- __reloadSaveMenu;
- __defaultDownloadFolder;
- __getTorrentInfo;
@end

@implementation DownloadSetupWindowController

//------------------------------------------------------------------------------
//		• init/dealloc
//------------------------------------------------------------------------------

- init
{
	self = [super initWithWindowNibName:@"DownloadSetupForFile"];

	if (self)
	{
		__pairs = [NSMutableDictionary new];
		__containerFormatter = [[SKContainerFormatter new] setShowsSummary:YES];	
	}
	
	return self;
}

- (void) dealloc
{
	[__emptyOptionsView release];
	[__emptyDetailsView release];
	
	[__pairs release];
	[__containerFormatter release];

	[__agent release];
	[__tomatoMetainfo release];
	[__outlineRoot release];
	
	[super dealloc];
}

//------------------------------------------------------------------------------
//		• windowDidLoad
//------------------------------------------------------------------------------

-(void) windowDidLoad
{
	__emptyOptionsView = [[__downloadOptionsBox contentView] retain];
	__emptyDetailsView = [[__detailsBox contentView] retain];
	
	[__fileNameTabView selectTabViewItemWithIdentifier:@"progress"];
	[[[__detailsOutline tableColumns] objectAtIndex:0] setIdentifier:@"key"];
	
	__saveMenuFrame = [__saveToMenu frame];
	__saveMenuSuperFrame = [[__saveToMenu superview] frame];
	
	[__fileNameText setStringValue:@""];	
		//just in case. Old code raised an exception once and the default text showed up.
}

//------------------------------------------------------------------------------
//		• showTorrentDownloadSetup
//------------------------------------------------------------------------------
//	activation point

- showTorrentDownloadSetup
{
	[self window];	//load stuff.

	__gotGo = YES;
	
	//the nib has become a scary place. set default enabled here.
	[__detailsToggleButton setEnabled:NO];
	[__downloadOptionsToggleButton setEnabled:NO];
	[__startButton setEnabled:NO];
	[__saveToMenu setEnabled:NO];
	
	[self __getTorrentInfo];
	[self __reloadSaveMenu];
	
	[[self window] center];
	[[self window] makeKeyAndOrderFront:nil];

	return self;
}



//------------------------------------------------------------------------------
//		• windowWillClose:
//------------------------------------------------------------------------------
//	saves the download options in user defaults.

#pragma mark -saveDownloadOptions

- (void) windowWillClose:notif
{
	if (__saveOptions)
	{
		id		settings = [[self __agent] downloadConfigurationDictionary];
		if (settings)
		{
			[[self window] endEditingFor:nil];
			[[NSUserDefaults standardUserDefaults] setObject:settings 
													  forKey:kUserDefault_TomatoAgent_downloadOptions];
			[PreferencesController noteTorrentDownloadOptionsDefaultsChanged];
		}
	}
	
	[self __setAutoStartTimer:nil];
}

- (BOOL) shouldCloseDocument
{
	return __gotGo == YES;
}

- closeWithoutClosingDocument
{
	__gotGo = NO;
	[self close];
	return self;
}

//------------------------------------------------------------------------------
//		• action methods
//------------------------------------------------------------------------------

- (IBAction)choseSaveTarget:(id)sender
{
	id		path = [[__saveToMenu selectedItem] representedObject];
	
	if (path)
		[[self document] setSaveDownloadPath:path];
	
	[self __setAutoStart:NO];
}

- (IBAction)saveOptions:(id)sender
{
	__saveOptions = [sender state] == NSOnState;
}

- (IBAction)startDownload:(id)sender
{
	__gotGo = NO;
	
	[[self window] endEditingFor:nil];

	if ([[self document] saveDownloadPath] == nil)
		[[self document] setSaveDownloadPath: [[__saveToMenu selectedItem] representedObject]];

	[[self document] startDownload:nil];

	[self __setAutoStartTimer:nil];
}

- (IBAction)toggleDetailsDisplay:(id)sender
{
	id	view = __emptyDetailsView;
	
	if ([sender state] == NSOnState)
		view = __torrentDetailsView;
		
	[self __setAutoStart:NO];
	[__detailsBox switchContentViewTo:view showingViewDuringResize:__emptyDetailsView];
	
	if (view == __torrentDetailsView)
		[__detailsOutline reloadData];
}

- (IBAction)toggleOptionsDisplay:(id)sender
{
	id				view = __emptyOptionsView;
	
	if ([sender state] == NSOnState) 
	{
		#if 1
		[__moduleOptionsController setModuleOptions: [[self __agent] downloadConfigurationDictionary]	
								  defaultsSelector: @selector(defaultDownloadConfigurationDictionary)
						 controllingTableForWindow: [self window]];
		#endif
		view = __downloadOptionsView;
	}
	
	[self __setAutoStart:NO];
	[__downloadOptionsBox switchContentViewTo:view showingViewDuringResize:__emptyOptionsView];
	
	if (view == __downloadOptionsView)
		[__moduleOptionsController reloadData];
}

//I guess I should do this too.
- (BOOL)validateMenuItem:menuItem;
{
	BOOL	result = NO;
	
	if ([menuItem action] == @selector(copy:))
	{
		id	firstResponder = [[self window] firstResponder];
		
		if (firstResponder == __detailsOutline)
		{
			if ([__detailsOutline selectedRow] >= 0)
				result = YES;
		}
	}
	
	return result;
}

- (void) copy:sender
{
	int	row = [__detailsOutline selectedRow];
	
	if (row >= 0)
	{
		id		item,
				columns, column,
				pasteboard,
				result;
		
		columns = [[__detailsOutline tableColumns] objectEnumerator];
		item = [__detailsOutline itemAtRow:row];
		result = [NSMutableArray array];
		
		if (item)
		{
			while (column = [columns nextObject])
			{
				[result addObject:[self outlineView: __detailsOutline
						  objectValueForTableColumn: column
											 byItem: item]];
			}
		}
		
		if ([result count] > 0)
		{
			pasteboard = [NSPasteboard generalPasteboard];
			
			[pasteboard declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:nil];
			
			[pasteboard setString: [result componentsJoinedByString:SKLocalizedString(@", ", @"comma space")]
						  forType: NSStringPboardType];
		}
	}
}

#define shoveToOutline_(key, name)	{if ([__tomatoMetainfo objectForKey:(key)]) [__outlineRoot setObject:[__tomatoMetainfo objectForKey:(key)] forKey:(name)];}

- populateOutline
{
	id		comment;
	
	__outlineRoot = [NSMutableDictionary new];
	
	shoveToOutline_(kTomatoAgentKey_metainfo_arg_nameFs, @"Name");
	shoveToOutline_(kTomatoAgentKey_metainfo_arg_announce, @"Announce");
	shoveToOutline_(kTomatoAgentKey_metainfo_arg_creationDate, @"creation date");
	//shoveToOutline_(kTomatoAgentKey_metainfo_arg_archiveContents, @"Files");

	if ([__tomatoMetainfo objectForKey:kTomatoAgentKey_metainfo_arg_archiveContents]) 
		[__outlineRoot setObject:[__tomatoMetainfo objectForKey:kTomatoAgentKey_metainfo_arg_archiveContents]
						  forKey:kTomatoAgentKey_metainfo_arg_archiveContents];
	
	if ((comment = [__tomatoMetainfo objectForKey:kTomatoAgentKey_metainfo_arg_comment]) && [(NSString*)comment length] > 0)
	{
		shoveToOutline_(kTomatoAgentKey_metainfo_arg_comment, @"Comment");
	}
	
	return self;
}

- setTomatoMetainfoDictionary:foo;
{
	id		failure = nil;
	
	[__detailsFetchProgress stopAnimation:nil];

	//if ([response fault] == nil)
	{
		failure = [foo objectForKey:kTomatoAgentKey_failure];
		[__fileNameTabView selectTabViewItemWithIdentifier:@"name"];
		
		if (failure == nil)
		{
			id		info;
			id		display;
			id		sizeString;
			id		sizeFormat;
			id		comment;
			
			info = __tomatoMetainfo = [[foo objectForKey:@"metainfo"] retain];
			[self populateOutline];

			//the name text.
			display = SKLocalizedString(@"%@ - %@", @"(name) — (size format); torrent info");
			
			sizeString = [[NSApp delegate] formatDataSize:[__tomatoMetainfo objectForKey:kTomatoAgentKey_metainfo_arg_totalBytes]];
			sizeFormat = @"%@";
			
			if ([[__tomatoMetainfo objectForKey:kTomatoAgentKey_metainfo_arg_isArchive] boolValue] == YES)
				sizeFormat = SKLocalizedString(@"%@ (Folder Archive)", @"(size) folder archive - torrent info");
			
			display = [NSString stringWithFormat:display, 
				[info objectForKey:kTomatoAgentKey_metainfo_arg_nameFs],
				[NSString stringWithFormat:sizeFormat, sizeString]];
			
			[__fileNameText setStringValue:display];
			
			//comment.
			if ((comment = [__tomatoMetainfo objectForKey:kTomatoAgentKey_metainfo_arg_comment]) && [(NSString*)comment length] > 0)
			{
				unsigned int		saved;
				
				saved = [__saveToBox autoresizingMask];
				[__saveToBox setAutoresizingMask:NSViewMaxYMargin];
				
				[__commentText setStringValue:comment];
				[__commentBox switchContentViewTo:__commentView showingViewDuringResize:nil];
				
				[__saveToBox setAutoresizingMask:saved];
			}
			
			//load outline data.
			[__detailsOutline reloadData];
			
			[__detailsToggleButton setEnabled:YES];
			[__downloadOptionsToggleButton setEnabled:YES];
			[__startButton setEnabled:YES];
			[__saveToMenu setEnabled:YES];
			
			[self __setAutoStart:YES];
			
			[self __reloadSaveMenu];
		}
		else
		{
			[__fileNameText setStringValue:[NSString stringWithFormat:@"%@%@",
				SKLocalizedString(@"Can't use the torrent file: ", @"torrent failure title (desc)"),
				SKLocalizedString([[failure objectAtIndex:0] objectForKey:kTomatoAgentKey_failure_desc], nil)]];
			
			[self __setAutoStart:NO];
		}
	}
	//else
	//	NSLog(@"fault getting metainfo: %@", [response fault]);
	
	return self;
}

@end

@implementation DownloadSetupWindowController (Private)
- __agent
{
	if (__agent == nil)
		__agent = [[[self document] tomatoTorrentAgent] retain];
	return __agent;
}

- __setAutoStartTimer:timer;
{
	if (__autoStartTimer)
	{
		[__autoStartTimer invalidate];
		[__autoStartTimer release];
	}
	
	__autoStartTimer = [timer retain];
	
	return self;
}

- __setAutoStart:(BOOL)start
{
	[self __setAutoStartTimer:nil];

	if (start)
	{
		NSTimeInterval	interval;
		
		interval = [[NSUserDefaults standardUserDefaults] floatForKey:kUserDefault_downloadAutostartSecs];
		if (interval > 0)
		{
			id timer;
			timer = [NSTimer scheduledTimerWithTimeInterval:interval
													 target:self
												   selector:@selector(__startDownloadFromTimer:)
												   userInfo:nil
													repeats:NO];
			[self __setAutoStartTimer:timer];
			
			{
				id		format;
				id		minuteFormat;
				int		numMinutes;
				
				numMinutes = interval/60;
				if (numMinutes == 0) numMinutes = 1;
				
				if (numMinutes > 1) 
					minuteFormat = SKLocalizedString(@"%d minutes", @"autostart num minutes fmt (integer num minutes)");
				else				
					minuteFormat = SKLocalizedString(@"one minute", @"autostart num minutes fmt (integer num minutes is 1)");
				
				minuteFormat = [NSString stringWithFormat:minuteFormat, numMinutes];
				format = SKLocalizedString(@"The download will start automatically in less than %@.", 
					@"autostart desc (#autostart num minutes fmt)");
				
				[__autostartTitle setStringValue:[NSString stringWithFormat:format, minuteFormat]];
			}
			
			[__startButton setTitle:SKLocalizedString(@"Start Now", @"autostart start download now button")];
		}
	}
	else
	{
		[__autostartTitle setStringValue:
			SKLocalizedString(@"Click Start to begin downloading.", @"file setup download no autostart")];

		[__startButton setTitle:SKLocalizedString(@"Start", @"autostart off start download button")];
	}
		
	return self;
}

//------------------------------------------------------------------------------
//		• __reloadSaveMenu
//------------------------------------------------------------------------------
//	The save menu has the download folder first, then the user-defined download
//	path if any, and then a 'save as' menu item to set the user-defined download path.

- __reloadSaveMenu;
{
	id		menu, 
			item,
			downloadFolder,
			recentDownloadFolders,
			documentDestPath,
			saveAsTitle,
			fileManager,
			torrentFilename;
	SEL		saveAsSel;
	bool	haveDocumentPath = NO;
	
	fileManager = [NSFileManager defaultManager];
	torrentFilename = [__tomatoMetainfo objectForKey:kTomatoAgentKey_metainfo_arg_nameFs];
	[__saveToMenu removeAllItems];
	menu = [__saveToMenu menu];
	[menu setAutoenablesItems:NO];

	downloadFolder = [self __defaultDownloadFolder];
	
	//	add the user defined download folder, if any.
	//
	documentDestPath = [[self document] saveDownloadPath];
	if (documentDestPath != nil && [documentDestPath isEqual:downloadFolder] == NO)
	{
		id		title = [fileManager displayNameAtPath:documentDestPath];
		
		title = [title stringByAppendingFormat:@" - %@", 
			[fileManager displayPathForPath:[documentDestPath stringByDeletingLastPathComponent]]];
		
		item = [[[NSMenuItem alloc] initWithTitle:title
										   action:nil 
									keyEquivalent:@""] autorelease];
		
		[item setRepresentedObject:documentDestPath];
		[menu addItem:item];
		[__saveToMenu selectItem:item];

		[menu addItem:[NSMenuItem separatorItem]];
		
		haveDocumentPath = YES;
	}

	//	add internet config item
	//
	item = [[[NSMenuItem alloc] initWithTitle:[fileManager displayPathForPath:downloadFolder]
									   action:nil 
								keyEquivalent:@""] autorelease];
	
	[item setRepresentedObject:[downloadFolder stringByAppendingPathComponent:torrentFilename]];
	[menu addItem:item];
	if (!haveDocumentPath) [__saveToMenu selectItem:item]; 	
	
	//	add recent download folders
	//
	
	recentDownloadFolders = [DownloadDocument recentTorrentDownloadFolders];
	if ([recentDownloadFolders count] > 0)
	{
		id		ite, obj;
		
		ite = [recentDownloadFolders objectEnumerator];
		while (obj = [ite nextObject])
		{
			if ([obj isEqual:downloadFolder] == NO)
			{
				id		title = [fileManager displayPathForPath:obj];
				
				item = [[[NSMenuItem alloc] initWithTitle:title
												   action:nil 
											keyEquivalent:@""] autorelease];
				
				[item setRepresentedObject:[obj stringByAppendingPathComponent:torrentFilename]];
				[menu addItem:item];
			}
		}
	}
	
	//	save as menu item
	//

	[menu addItem:[NSMenuItem separatorItem]];

	saveAsTitle = SKLocalizedString(@"Save As...", nil);
	saveAsSel = @selector(__chooseDownloadPath:);
	
	if (__tomatoMetainfo)
	{
		if ([[__tomatoMetainfo objectForKey:kTomatoAgentKey_downloadState_isDirectory] intValue] != 0)
		{
			saveAsTitle = SKLocalizedString(@"Download To Folder...", nil);
			saveAsSel = @selector(__chooseDownloadFolder:);
		}
	}
	
	item = [[[NSMenuItem alloc] initWithTitle:saveAsTitle
									   action:saveAsSel
								keyEquivalent:@""] autorelease];
	[item setTarget:self];
	[menu addItem:item];
	
	//ok. It looks stupid for case #2, where the file name is followed by its containing folder.
	//I have to resize the menu, and then resize and reposition its superview.
	{
		NSRect		windowFrame;
		NSRect		newFrame;
		NSRect		oldSuperframe;
		NSRect		newSuperframe;
		float		offset;
		
		//windowFrame = [[[self window] contentView] frame];
		windowFrame = [[[__saveToMenu superview] superview] frame];	//better.
		oldSuperframe = [[__saveToMenu superview] frame];
		
		[__saveToMenu sizeToFit];
		newFrame = [__saveToMenu frame];
		
		offset = newFrame.size.width - __saveMenuFrame.size.width;
		
		newSuperframe = __saveMenuSuperFrame;
		newSuperframe.origin.y = oldSuperframe.origin.y;

		newSuperframe.origin.x -= offset/2;
		newSuperframe.size.width += offset;
		
		if (newSuperframe.origin.x < windowFrame.origin.x)
		{
			offset = windowFrame.origin.x - newSuperframe.origin.x;
			
			newSuperframe.origin.x = windowFrame.origin.x;
			newSuperframe.size.width = windowFrame.size.width;
			
			newFrame.size.width -= offset*2;
			[__saveToMenu setFrame:newFrame];
		}
		
		[[__saveToMenu superview] setFrame:newSuperframe];
		
			//mostly important when the new frame is smaller. The difference
			//area will not be updated otherwise.
		[[[__saveToMenu superview] superview] setNeedsDisplay:YES];//InRect:oldSuperframe];
	}
	
	return self;
}

//------------------------------------------------------------------------------
//		• __defaultDownloadPath
//------------------------------------------------------------------------------

- __defaultDownloadFolder
{
	id	result;

	result = [[SKInternetConfig instance] downloadFolderPath];
	
	if (result == nil) 
		result = NSHomeDirectory();		//+++ the desktop folder is better, 
										//because it's got more restricted access

	return result;
}

//------------------------------------------------------------------------------
//		• __getTorrentInfo
//------------------------------------------------------------------------------

- __getTorrentInfo
{
	/*	The document is doing this for us already.
	[[self __agent] metainfoPath:[[self document] fileName] delegate:self];
	
	if ([__detailsFetchProgress respondsToSelector:@selector(setStyle:)])
		[__detailsFetchProgress setStyle:NSProgressIndicatorSpinningStyle];*/

	[__detailsFetchProgress startAnimation:nil];
	
	return self;
}

//------------------------------------------------------------------------------
//		• __chooseDownloadPath:
//------------------------------------------------------------------------------

- (void) __chooseDownloadPath:sender
{
	id	openPanel = [NSSavePanel savePanel];
	
	[self __setAutoStart:NO];
	
	[openPanel beginSheetForDirectory:[self __defaultDownloadFolder] 
								 file:[__tomatoMetainfo objectForKey:kTomatoAgentKey_metainfo_arg_nameFs]
					   modalForWindow:[self window]
					    modalDelegate:self 
					   didEndSelector:@selector(__userChoseFileUsing:returnCode:contextInfo:) 
						  contextInfo:nil];
}

- (void) __chooseDownloadFolder:sender
{
	id	openPanel = [NSOpenPanel openPanel];
	
	[self __setAutoStart:NO];

	[openPanel setCanChooseDirectories:YES];
	[openPanel setCanChooseFiles:NO];
	[openPanel setPrompt:SKLocalizedString(@"Choose", @"choose folder prompt")];
	
	[openPanel beginSheetForDirectory:[self __defaultDownloadFolder] 
								 file:nil
								types:nil
					   modalForWindow:[self window]
					    modalDelegate:self 
					   didEndSelector:@selector(__userChoseFileUsing:returnCode:contextInfo:) 
						  contextInfo:nil];
}

- (void)__userChoseFileUsing:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
	if (returnCode == NSOKButton)
	{
		[[self document] setSaveDownloadPath:[sheet filename]];
	}
	
	[self __reloadSaveMenu];
	[sheet orderOut:nil];
}

- (void) __startDownloadFromTimer:(NSTimer*)timer
{
	if ([__startButton isEnabled])
		[self startDownload:nil];

	else
		NSLog(@"auto start timer with the button disabled? Not starting.");
}


@end

//------------------------------------------------------------------------------
//		• DownloadSetupWindowController (TorrentDetailsOutlineView)
//------------------------------------------------------------------------------
		/// function for sorting localized keys.
		//
		//		it was an inner function but compiler bugs caused it to crash for some
		//		people. Hopefully there aren't any more, I use these a lot :P
		
	static NSComparisonResult compare_localized(id a, id b, void *context)
	{
		NSComparisonResult	result = NSOrderedSame;
		id					a_key = [a objectAtIndex:kItemKey];
		id					b_key = [b objectAtIndex:kItemKey];
		static Class		nsstringClass = Nil;
		
		if (nsstringClass == Nil) nsstringClass = [NSString class];
		
		if ([a_key isKindOfClass:nsstringClass] && [b_key isKindOfClass:nsstringClass])
		{
			id	a_local = TTAgentLocalizedString_(a_key, kTomatoAgentStringsTable_metainfo);
			id	b_local = TTAgentLocalizedString_(b_key, kTomatoAgentStringsTable_metainfo);
			
			result = [a_local localizedCompare:b_local];
		}
		
		return result;
	}

@implementation DownloadSetupWindowController (TorrentDetailsOutlineView)

- __pairForIndex:(int)index ofItem:item
{
	id		parent,
			parentKey;
	id		pairKey,
			children,
			pair;

	if (item == nil) 
		parent = __outlineRoot,
		parentKey = kRootKey;
		
	else
	{
		id keyString = [[item objectAtIndex:kItemKey] description];
		
		parent = [item objectAtIndex:kItemValue];
		parentKey = [[item objectAtIndex:kItemParentPathKey] stringByAppendingPathComponent:keyString];
	}
		
	pairKey = [NSValue valueWithPointer:parent];
	children = [__pairs objectForKey:pairKey];

	if (children == nil)
	{
		id		keys 	= [parent keyEnumerator];	//skrange
		id		objects = [parent objectEnumerator];
		id		key, 
				object;		
		
		children = [NSMutableArray arrayWithCapacity:[parent count]];
		
		while (object = [objects nextObject], key = [keys nextObject])
		{
			[children addObject:[NSArray arrayWithObjects:key, object, parentKey, nil]];
		}
		
		[children sortUsingFunction:compare_localized context:nil];

		[__pairs setObject:children forKey:pairKey];
	}

	pair = [children objectAtIndex:index];
	
	return pair;
}

- (BOOL) __outlineItemIsFilePathEntry:item
{
	NSRange		pathRange;
	id			parentKeyPath 	= [item objectAtIndex:kItemParentPathKey];
	id			key 			= [item objectAtIndex:kItemKey];
	
	pathRange = [parentKeyPath rangeOfString:@"/archiveContents"];
	
	return pathRange.location == 0 && [key isKindOfClass:[NSNumber class]];
}

- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item;
{
	return [self __pairForIndex:index ofItem:item];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item;
{
	if ([self __outlineItemIsFilePathEntry:item])
		return NO;
	
	return [[item objectAtIndex:kItemValue] respondsToSelector:@selector(keyEnumerator)];
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item;
{
	int		result = 0;
	
	if (__outlineRoot) 
	{
		if (item == nil) 	result = [__outlineRoot count];
		else 				result = [[item objectAtIndex:kItemValue] count];
	}
	
	return result;
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item;
{
	id		result 			= nil;
	id		key 			= [item objectAtIndex:kItemKey],
			value 			= [item objectAtIndex:kItemValue];
	BOOL	isKeyColumn		= [[tableColumn identifier] isEqual:@"key"];

	if ([self __outlineItemIsFilePathEntry:item])
	{
		//	files listing in info.
		//
		//	put the file size in the key column because it's got short width,
		//	and put the path in the value column.
		if (isKeyColumn)
		{
			result = [[NSApp delegate] formatDataSize:
				[value objectForKey:kTomatoAgentKey_metainfo_arg_archiveContents_size]];
		}
		else
		{
			result = [value objectForKey:kTomatoAgentKey_metainfo_arg_archiveContents_path];
		}
	}
	else
	{
		if (isKeyColumn)
			result = TTAgentLocalizedString_(key, kTomatoAgentStringsTable_metainfo);
		
		else if ([value respondsToSelector:@selector(count)] == NO)
			result = [[NSApp delegate] stringForObjectValue:value forKey:key];

		else
			result = [__containerFormatter stringForObjectValue:value];
	}
	
	return result;
}
@end
