//==============================================================================
//		FetchURLWindowController.m					Tomato
//==============================================================================
//  Created by Sarwat Khan on 30/04/05.
//  Copyright (c) 2005 Sarwat Khan. All rights reserved.

#import "AppDelegate.h"
#import "TomatoTorrentAgent.h"

#import "FetchURLWindowController.h"

#define kIdleState		0
#define kFetchingState	1
#define kVerifyingState	2

#include <unistd.h>

static id g_fetchurlcontroller = nil;

@interface FetchURLWindowController (Private)
- __fetchURLFromString:string;
- __finishedLoadingData:data;
- (BOOL) __isFetchingURL;
- __displayStatusString:string;
- __loadingStarted;
- __loadingStopped;
- __cancelLoad;
@end

@implementation FetchURLWindowController
+ instance
{
	if (g_fetchurlcontroller == nil)
		g_fetchurlcontroller = [self new];
	
	return g_fetchurlcontroller;
}

+ fetchTorrentFromURLString:url;
{
	id		bugger = nil;
	
	//this is convoluted. 
	if (g_fetchurlcontroller == nil)
	{
		bugger = [self instance];
	}
	else if ([g_fetchurlcontroller __isFetchingURL])
	{
		bugger = [[self new] retain];	//see windowShouldClose:.
	}
	else
		bugger = g_fetchurlcontroller;
	
	[bugger fetchTorrentFromURLString:url];
	[bugger showWindow:nil];
	
	return bugger;
}

//---------------------------------------------------------------------------
//		• - init
//---------------------------------------------------------------------------

- init
{
	if (self = [super initWithWindowNibName:@"FetchURL"])
	{
		__agent = [TomatoTorrentAgent new];
	}
	
	return self;
}

- (void) dealloc
{
	if (g_fetchurlcontroller == self)
		g_fetchurlcontroller = nil;

	[self __loadingStopped];
	
	[__agent shutdown];
	[__agent release];
	
	[super dealloc];
}

//---------------------------------------------------------------------------
//		• - windowDidLoad
//---------------------------------------------------------------------------

- (void) windowDidLoad
{
	[[self window] center];
	[__fetchButton setTitle:SKLocalizedString(@"Fetch Torrent", @"Fetch Torrent")];
}

- (IBAction)fetchTorrentFromEnteredURL:(id)sender;
{
	//if ([self __isFetchingURL] == NO)
	if (__state == kIdleState)
	{
		[[self window] endEditingFor:__urlText];
		[self __fetchURLFromString:[__urlText stringValue]];
	}
	else
	{
		[self __cancelLoad];
	}
}

- fetchTorrentFromURLString:url
{
	[self window];
	
	if (![self __isFetchingURL])
		return [self __fetchURLFromString:url];

	return nil;
}

- (BOOL)windowShouldClose:(id)sender;
{
	bool	result = YES;
	
	if (g_fetchurlcontroller == self)
	{
		result = NO;
		[sender orderOut:nil];
	}
	else
		[self autorelease];	//see +fetch
	
	return result;
}

//---------------------------------------------------------------------------
//		• - URLHandleResourceDidBeginLoading:
//---------------------------------------------------------------------------

- (void)URLHandleResourceDidBeginLoading:(NSURLHandle *)sender;
{
}

- (void)URLHandle:(NSURLHandle *)sender resourceDataDidBecomeAvailable:(NSData *)newBytes;
{
	[__torrentData appendData:newBytes];
}

- (void)URLHandleResourceDidFinishLoading:(NSURLHandle *)sender;
{
	[self __finishedLoadingData:__torrentData];
}

- (void)URLHandleResourceDidCancelLoading:(NSURLHandle *)sender;
{
	[self __loadingStopped];
	[self __displayStatusString:@""];
}

- (void)URLHandle:(NSURLHandle *)sender resourceDidFailLoadingWithReason:(NSString *)reason;
{
	[self __loadingStopped];
	[self __displayStatusString:
		[NSString stringWithFormat:
			SKLocalizedString(@"Failed to load the URL: %@", @"URL:resourceDidFailLoadingWithReason:"),
			reason]];
	NSLog(@"%@", [NSString stringWithFormat:
		SKLocalizedString(@"Failed to load the URL: %@", @"URL:resourceDidFailLoadingWithReason:"),
		reason]);
}

@end


//---------------------------------------------------------------------------
//		• FetchURLWindowController
//---------------------------------------------------------------------------

@implementation FetchURLWindowController (Private)

//---------------------------------------------------------------------------
//		• - __fetchURLFromString:
//---------------------------------------------------------------------------

- __fetchURLFromString:string;
{
	id		url;
	
	if ([[__urlText stringValue] isEqual:string] == NO)
		[__urlText setStringValue:string];
	
	if ([(NSString*)string length] == 0) return nil;
	
	if (url = [NSURL URLWithString:string])
	{
		if (__urlHandle = [[[NSURLHandle URLHandleClassForURL:url] alloc] initWithURL:url cached:NO])
		{
			[self __loadingStarted];
			[__urlHandle retain];
			[__urlHandle addClient:self];
			[__urlHandle beginLoadInBackground];
			
			[__fetchButton setTitle:SKLocalizedString(@"Cancel", nil)];
		}
	}
	
	if (__urlHandle == nil)
	{
		[self __displayStatusString:
			SKLocalizedString(@"I cannot use that URL. Perhaps it was typed in wrong?", nil)];
	}
	
	return self;
}

- (BOOL)__isFetchingURL;
{
	return 	__state != kIdleState;
}

- __displayStatusString:string;
{
	[__statusText setStringValue:string];
	
	return self;
}

//---------------------------------------------------------------------------
//		• - __loadingStarted
//---------------------------------------------------------------------------
//	- sets fetch state
- __loadingStarted;
{
	[__progressSpinner startAnimation:nil];
	__torrentData = [NSMutableData new];
	__state = kFetchingState;
	return self;
}

//---------------------------------------------------------------------------
//		• - __finishedLoadingData:
//---------------------------------------------------------------------------
//	call before calilng loadingStopped.
//
//	- removes fetch state
- __finishedLoadingData:data;
{
	id		savePath;
	char	*thing;
	id		file;
	
	savePath = [NSTemporaryDirectory() stringByAppendingPathComponent:@"fetchedMetainfo.XXXX"];

	thing = strdup([savePath fileSystemRepresentation]);
	mktemp(thing);

	file = [NSString stringWithUTF8String:thing];
	free(thing);
	
	if (thing)
	{
		[data writeToFile:file atomically:NO];

		[__agent metainfoPath:file delegate:self];
		[__tempSavePath release];
		__tempSavePath = [file copy];
		
		[self __displayStatusString:SKLocalizedString(@"Verifying torrent...", nil)];
		__state = kVerifyingState;
	}
	else
	{
		[self __loadingStopped];	//*** handles idle state
		[self __displayStatusString:SKLocalizedString(@"Couldn't save the torrent data! Sorry.", nil)];
	//	__state = kIdleState;
	}

	return self;
}

//---------------------------------------------------------------------------
//		• - __loadingStopped
//---------------------------------------------------------------------------
//	Only way to stop the spinner. __urlHandle and __data are invalidated.
//
//	- sets idle state
- __loadingStopped;
{
	__state = kIdleState;
	[__urlHandle release];
	__urlHandle = nil;

	[__torrentData release];
	__torrentData = nil;
	
	[__tempSavePath release];
	__tempSavePath = nil;
	
	[__progressSpinner stopAnimation:nil];
	
	[__fetchButton setTitle:SKLocalizedString(@"Fetch Torrent", @"Fetch Torrent")];
	[__statusText setStringValue:@""];
	
	return self;
}

- __cancelLoad
{
	if (__urlHandle) [__urlHandle cancelLoadInBackground];
	[self __loadingStopped];	//*** handles idle
	
	return self;
}

- tomatoTorrentAgent:agent metainfo:response;
{
	id		content = [response responseContent];
	id		statusMessage = @"";

	if (__tempSavePath && [__tempSavePath isEqual:[content objectForKey:kTomatoAgentKey_metainfo_sourceFile]])
	{
		if ([content objectForKey:@"failure"] == nil)
		{
			id		torrentPath, doc;
			id		fileManager = [NSFileManager defaultManager];
			id		metainfo = [content objectForKey:kTomatoAgentKey_metainfo_arg];
			
			//move the bugger to the caches directory.
			torrentPath = [[NSApp delegate] cachesDirectoryPathForCacheKey:@"Fetched Torrents"];
			torrentPath = [torrentPath stringByAppendingPathComponent:
				[NSString stringWithFormat:@"%@.torrent", 
					[metainfo objectForKey:kTomatoAgentKey_metainfo_arg_nameFs]]];
			
			[fileManager removeFileAtPath:torrentPath handler:nil];
			if (![fileManager movePath:__tempSavePath toPath:torrentPath handler:nil])
				torrentPath = __tempSavePath;
			
			if (doc = [[NSDocumentController sharedDocumentController] openDocumentWithContentsOfFile:torrentPath display:YES])
			{
				[doc showWindows];
				
				if (g_fetchurlcontroller == self)
					[[self window] orderOut:nil];
				else
					[self close];
			}
			else
				statusMessage = SKLocalizedString(@"Sorry, I couldn't use that torrent", nil);
		}
		else
		{
			NSLog(@"failure: %@", [content objectForKey:@"failure"]);
			statusMessage = SKLocalizedString(@"I was not able to download a valid torrent file", nil);
		}
		
		[self __loadingStopped];
		[self __displayStatusString:statusMessage];
	}
	//else not applicable to self.
	
	return self;
}

@end

