//
//  PrimaryIPv4Address.m
//  Tomato
//
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khan's name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "PrimaryIPv4Address.h"
#import "NSBundle+MainBundleExtras.h"

static void
PrimaryIPv4Addr_SCDynamicStoreUpdated(SCDynamicStoreRef	store, CFArrayRef changedKeys, void *info);

@implementation PrimaryIPv4Address
- init
{
	[super init];
	
	_store = SCDynamicStoreCreate(NULL, (CFStringRef)[[NSBundle mainBundle] bundleIdentifier], PrimaryIPv4Addr_SCDynamicStoreUpdated, (void*)self);
	if (_store)
		//this doesn't do the job. I guess I don't know what it's supposed to do.
		SCDynamicStoreSetNotificationKeys(_store, (CFArrayRef)[NSArray arrayWithObject:@"State:/Network/Global/IPv4"], NULL);
	return self;
}

- (void) dealloc
{
	CFRelease(_store);
	[super dealloc];
}

- IPv4String
{
	id		result = @"127.0.0.1";
	id		state, interface, ipv4, addrs;
	
	//get the name of the interface, which is like en0.
	if (state = (id)SCDynamicStoreCopyValue(_store, (CFStringRef)@"State:/Network/Global/IPv4"))
	{
		if (interface = [state objectForKey:@"PrimaryInterface"])
		{
			if (ipv4 = (id)SCDynamicStoreCopyValue(_store, (CFStringRef)
				[NSString stringWithFormat:@"State:/Network/Interface/%@/IPv4", interface]))
			{
				if (addrs = [ipv4 objectForKey:@"Addresses"])
				{
					if ([addrs count] > 0)
					{
						result = [[[addrs objectAtIndex:0] retain] autorelease];
					}
				}
				
				[ipv4 release];
			}
		}
		
		[state release];
	}
	
	return result;
}

@end

static void
PrimaryIPv4Addr_SCDynamicStoreUpdated(SCDynamicStoreRef	store, CFArrayRef changedKeys, void *info)
{
	[[NSNotificationCenter defaultCenter] postNotificationName:kPrimaryIPv4AddressUpdatedNotification
														object:info];
}
