//
//  TorrentConnectionRateFormatter.m
//  BitTorrent
//
//  Created by Sarwat Khan on Sun Oct 26 2003.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khan's name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "TorrentConnectionRate.h"
#import <AppKit/AppKit.h>
#import "AppDelegate.h"
#import "PreferencesController.h"

#define kKilobyte       1024.0
#define kMegabyte       (1024 * kKilobyte)
#define kGigabyte       (1024 * kMegabyte)

static id format_rate_text(id rate, id total, id totalBytesForFile)
{
	id		display = nil;
	
	if (rate)
	{
		if (total)
		{
			id	bytesTotal = [NSNumber numberWithDouble:[total doubleValue]];
			
			if (totalBytesForFile == nil || ![PreferencesController doAutoStopTomatoTorrentDownloadsByShareRatio])
			{
				display = [NSString stringWithFormat:
					SKLocalizedString(@"%2.1f K/s (%@)", @"transfer rate, total"),
					[rate doubleValue] / 1024, [[NSApp delegate] formatDataSize:bytesTotal]];
			}
			else
			{
				display = [NSString stringWithFormat:
					SKLocalizedString(@"%2.1f K/s (%@, %d%%)", @"transfer rate, total"),
					[rate doubleValue] / 1024, 
					[[NSApp delegate] formatDataSize:bytesTotal],
					(int)([total doubleValue]/[totalBytesForFile doubleValue]*100)];
			}
		}
		else
			display = [NSString stringWithFormat:
				SKLocalizedString(@"%2.1f K/s", @"transfer rate"),
				[rate doubleValue] / 1024];
	}
	else if (total)
	{
		id	bytesTotal = [NSNumber numberWithDouble:[total doubleValue]];
		display = [NSString stringWithFormat:
			SKLocalizedString(@"Total: %@", @"transfer rate na, total"),
			[[NSApp delegate] formatDataSize:bytesTotal]];
	}
				
	return display;
}

@implementation TorrentConnectionRateFormatter

//  •   awakeFromNib
//
//  This is controller-ish. x

- (void) awakeFromNib
{
	//listen to notifications for reload
	
	//
}

- (NSString *)stringForObjectValue:(id)obj;
{
	if ([obj isKindOfClass:[TorrentConnectionRate class]])
	{
		id		display = @"";
		id		upRate, downRate, upTotal, downTotal, activity;
		id		downloadText = nil, uploadText = nil;
		
		upRate = [obj valueForKey:@"upRate"];
		downRate = [obj valueForKey:@"downRate"];
		upTotal = [obj valueForKey:@"upTotal"];
		downTotal = [obj valueForKey:@"downTotal"];
		activity = [obj valueForKey:@"activity"];
		
		downloadText = format_rate_text(downRate, downTotal, nil);
		uploadText = format_rate_text(upRate, upTotal, [obj valueForKey:@"totalBytes"]);
		
		if ([activity isEqual:@"downloading"])
		{
			if (downloadText && uploadText)
				display = [NSString stringWithFormat:
					SKLocalizedString(@"Down: %@ * Up: %@", @"TorrentConnectionRateFormatter both"),
					downloadText, uploadText];

			else if (downloadText)
				display = [NSString stringWithFormat:
					SKLocalizedString(@"Download: %@", @"TorrentConnectionRateFormatter download text"),
					downloadText];
			
			else if (uploadText)
				display = [NSString stringWithFormat:
					SKLocalizedString(@"Upload: %@", @"TorrentConnectionRateFormatter upload text"),
					uploadText];
		}
		else if ([activity isEqual:@"seeding"]) 
		{
			if (uploadText)
				display = [NSString stringWithFormat:
					SKLocalizedString(@"Upload: %@", @"TorrentConnectionRateFormatter upload text"),
					uploadText];
		}
		return (display ? display : @"");
	}
	else
		return [obj description];
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error;
{
	return NO;
}
@end

@implementation TorrentConnectionRate

- copyWithZone:(NSZone*)zone
{
	return [self retain];   //this object is immutable. really. accessors are for initialization.
}

- (void) dealloc
{
	[_upTotal release];
	[_upRate release];
	[_upRate2 release];
	[_downTotal release];
	[_downRate release];
	[__activity release];
	[__totalBytes release];
	[super dealloc];
}

//these methods are named like this for key-value coding.

- upTotal;
{
	return _upTotal;
}

- setUpTotal:upTotal;
{
	[_upTotal release];
	_upTotal = [upTotal retain];
	return self;
}

- upRate2;
{
	return _upRate;
}

- setUpRate2:upRate;
{
	[_upRate release];
	_upRate = [upRate retain];
	return self;
}

- upRate;
{
	return _upRate;
}

- setUpRate:upRate;
{
	[_upRate release];
	_upRate = [upRate retain];
	return self;
}

- downRate;
{
	return _downRate;
}

- setDownRate:downRate;
{
	[_downRate release];
	_downRate = [downRate retain];
	return self;
}

- downTotal;
{
	return _downTotal;
}

- setDownTotal:downTotal;
{
	[_downTotal release];
	_downTotal = [downTotal retain];
	return self;
}

- setActivity:activity
{
	[__activity release];
	__activity = [activity retain];
	return self;
}

- activity
{
	return __activity;
}

- setTomatoDownloadSizeBytes:thing
{
	[__totalBytes release];
	__totalBytes = [thing retain];
	return self;
}

- TomatoDownloadSizeBytes
{
	return __totalBytes;
}

- totalBytes
{
	return __totalBytes;
}

- (void)handleTakeValue:(id)value forUnboundKey:(NSString *)key;
{
}

@end
