//==============================================================================
//		TorrentDownloadGetURLScriptCommand.m					TomatoTorrent
//==============================================================================
//  Created by Sarwat Khan on Sat Jun 07 2003.
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khans name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import "TorrentDownloadGetURLScriptCommand.h"

#import "DownloadDocument.h"

@implementation TorrentDownloadGetURLScriptCommand
- (id)performDefaultImplementation
{
	id		destination, url, saveTo, saveToFolder, 
			document = nil;
	OSErr	error = noErr;
	id		errorDesc = nil,
			errorData = nil;

	saveTo = [[self arguments] objectForKey:@"intoFile"];
	saveToFolder = [[self arguments] objectForKey:@"intoFolder"];
	
	if (destination = [self directParameter])
	{
		if (url = [NSURL URLWithString:destination])
		{
			if (document = [[NSDocumentController sharedDocumentController] 
				openDocumentWithContentsOfURL:url display:YES])
			{
				[document showWindows];
				
				if (saveTo)
				{
					[document setSaveDownloadPath:[saveTo stringByStandardizingPath]];
					[document startDownload:nil];
				}
				else if (saveToFolder)
				{
					[document setSaveDownloadDirectoryPath:[saveToFolder stringByStandardizingPath]];
					[document startDownload:nil];
				}
			}
			else
				error = errOSACantCreate,
				errorDesc = SKLocalizedString(@"Can't make torrent document with %@", @"get url: can't make document"),
				errorData = url;
		}
		else
			error = errOSACantCoerce,
			errorDesc = SKLocalizedString(@"Can't make %@ into a URL", @"get url: can't make (given) into a url."),
			errorData = destination;
	}
	else
		error = OSAMissingParameter, 
		errorDesc = SKLocalizedString(@"No URL specified", @"get url: no direct parameter");
	
	if (error)
	{
		[self setScriptErrorNumber:error];
		[self setScriptErrorString:[NSString stringWithFormat:errorDesc, errorData]];
	}
	
	return document;	//not scriptable in a useful way, but will be eventually.
}
@end
