//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khans name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import <Cocoa/Cocoa.h>

#include <sys/wait.h>
#include <unistd.h>

void sk_kill_group_sigfunc(int sig);
void sk_wait_group_sigfunc(int sig, siginfo_t *info, void *context);
void sk_hangup_on_group_exitfunc();

static BOOL		g_atexit = NO;		//if we hit the atexit kill() call
static BOOL		g_groupkill = NO;	//if we hit the sigfunc kill() call
static char* 	g_copyright = "Copyright (c) 2003, Sarwat Khan. All rights reserved.";

int main(int argc, const char *argv[])
{
	struct sigaction wait_action;

	g_copyright = g_copyright;

	setpgid(0, getpid());		//the Finder doesn't change the process group when it
								//launches stuff, so otherwise when you quit this app
								//you also get booted out of your login session.
								//
								//I'd actually like to set the pgid to the pid of the
								//first process I spawn, but they get spawned through threads
								//so I'd have to use a mutex to do that stuff, so I don't
								//feel like implementing that.

	signal(SIGHUP, sk_kill_group_sigfunc);
	signal(SIGINT, sk_kill_group_sigfunc);
	signal(SIGPIPE, sk_kill_group_sigfunc);
	signal(SIGTERM, sk_kill_group_sigfunc);
	
	signal(SIGSEGV, sk_kill_group_sigfunc);
	signal(SIGQUIT, sk_kill_group_sigfunc);
	signal(SIGILL, sk_kill_group_sigfunc);
	signal(SIGTRAP, sk_kill_group_sigfunc);
	signal(SIGEMT, sk_kill_group_sigfunc);
	signal(SIGFPE, sk_kill_group_sigfunc);
	signal(SIGBUS, sk_kill_group_sigfunc);
	signal(SIGSYS, sk_kill_group_sigfunc);
	
	wait_action.sa_sigaction = sk_wait_group_sigfunc;
	wait_action.sa_flags = SA_NOCLDSTOP;
	wait_action.sa_mask = 0;
	
	sigaction(SIGCHLD, &wait_action, NULL);

	/*
		NSApplicationMain never returns. Neither does calling
		[NSApp run]. I want to make sure that I send a SIGHUP to any processes
		left around, so I'll install a thingy to get it done. I don't want to
		do it from AppDelegate, I'd rather keep this logic in main().
		
		My guess is that NSApplicationMain calls exit(0) to properly set the
		exit status.
	*/
	
	atexit(sk_hangup_on_group_exitfunc);
	NSApplicationMain(argc, argv);
	
	return 0;
}

void 
sk_kill_group_sigfunc(int sig)
{
	g_groupkill = YES;
	
	if (g_atexit == NO)		//we're included in the kill group from atexit.
	{							//signal handlers don't re-enter, so the following killpg
								//won't fire this function.
		killpg(0, SIGTERM);
			
		switch (sig)
		{
			case SIGHUP:
			case SIGINT:
			case SIGPIPE:
			case SIGTERM:
				exit(1);
			break;
			
			default:
				abort();
			break;
		}
	}
}

void
sk_wait_group_sigfunc(int sig, siginfo_t *info, void *context)
{
	if (sig == SIGCHLD)
	{
		while (waitpid(WAIT_MYPGRP, NULL, WNOHANG) > 0)
			;
	}
}

void
sk_hangup_on_group_exitfunc()
{
	g_atexit = YES;
	
	if (g_groupkill == NO)
		killpg(0, SIGHUP);
}

//		now that I've added the atexit which is always called (normal or abnormal
//		shutdown), I could probably remove the signal handlers. The only good thing
//		about the signal handlers is that they send a SIGTERM instead of SIGHUP, 
//		which means that the agent could do something different in response (the 
//		agent receiving SIGTERM means abnormal shutdown of parent process. SIGHUP
//		just makes sure that any agents eventually exit).
