#!/usr/bin/env python
#
# Copyright (c) 2003, Sarwat Khan. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#         
#     1. Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#         
#     2. Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following disclaimer
#        in the documentation and/or other materials provided with the
#        distribution.
#         
#     3. Neither Sarwat Khan's name nor the names of any contributors to
#        the BitTorrentAgent Software may be used to endorse or promote
#        products derived from this software without specific prior written
#        permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
"""
A wrapper for the BitTorrent python modules. Receives commands line-by-line, and produces XML as output.

See the Agent module for more info.

"""

import sys, os
import signal, syslog
from TomatoTorrent import agent
import BitTorrent.ConvertedMetainfo

copyright = "Copyright (c) 2005, Sarwat Khan."


g_debug_level = 1

def log_message(x): 
	if g_debug_level >= 0: syslog.syslog(x)
	if g_debug_level >= 1: print >>sys.stderr, x

def die_later(sig, stack):
	signal.alarm(15)
	# sent by the agent lib after sending a shutdown message. extra safety.
	# also sent when the client app quits (threads may not have been able to send
	# the shutdown message).
	
def die(sig, stack):

	signal.alarm(10)
	
	if sig == signal.SIGTERM:
		raise Exception("SIGTERM")
		
	raise Exception("SIG - " + str(sig))


##	
##	main
##
##
def main():
	agent.setLoggingFunction(log_message)
	BitTorrent.ConvertedMetainfo.set_filesystem_encoding('utf-8', None)
	
	log_message("%s startup self pid: %s, group:%s" % (sys.argv[0], os.getpgrp(), os.getpid()))
	signal.signal(signal.SIGTERM, die)
	signal.signal(signal.SIGHUP, die_later)

	# should probably read args to change from stdin/stdout.
	requests = sys.stdin
	responses = sys.stdout
	sys.stdout = sys.stderr
	
	result = agent.Agent(requests, responses).run()
	
	if result:
		try:
			syslog.syslog("BitTorrent task exiting abnormally, %s" % result)
		except:
			pass
		
	log_message("%s exiting" % (sys.argv[0]))
	responses.write("%010d done.\n" % (0))
	
	die_later(None, None)		#threads may still be running

if __name__ == '__main__':
	main()
