#! /usr/bin/env python2.3
#  Copyright (c) 2003, Sarwat Khan. All rights reserved.
#  
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are
#  met:
#          
#      1. Redistributions of source code must retain the above copyright
#         notice, this list of conditions and the following disclaimer.
#          
#      2. Redistributions in binary form must reproduce the above
#         copyright notice, this list of conditions and the following disclaimer
#         in the documentation and/or other materials provided with the
#         distribution.
#          
#      3. Neither Sarwat Khan's name nor the names of any contributors to
#         the BitTorrent Macintosh software may be used to endorse or promote
#         products derived from this software without specific prior written
#         permission.
#  
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

g_sk_copyright = "Copyright (c) 2005, Sarwat Khan. All rights reserved."

from distutils import sysconfig
import sys, os, fnmatch, glob, pickle

kRootFolder = "TomatoTorrentAgent"
kInterpretersFolder = "Interpreters"
kResourcesFolder = "Resources"

kPickleDictionaryName = 'defined.pickled'
kPythonInfoPlistName = 'pythonInfo.plist'

description="""
DefinePythonInterpreters

Requires at least Python 2.2.

For each version of python  you want to support, run this script
with that binary of python. For example,

	/usr/bin/python2.2 %(command)s define-interpreter --no-zip
	/usr/local/bin/python2.3 %(command)s define

(define-intp is short for define-interpreter)

This will create the necessary data in ./%(interpDir)s/, which
are used by the Tomato.xcode to define the python binaries that
are used to compile the Python sources into optimized foo.

A dictionary describing the directory is maintained as a pickled file,
so if you muck around with the directory contents without using this script,
I suggest you junk the directory and start over. A plist version of the 
dictionary is written to ./%(interpDir)s/%(plistInfo)s if either the
plistservices or plistlib modules are available for the running interpreter.

Commands,
	define|define-interpreter [--no-zip]
	
		When run with a version of a python interpreter, will define
		that python version as an optimized bugger for BitTorrent Mac.
		--no-zip will make it not use a zip file, which causes a larger
		build targer, but somewhat faster launching for each agent 
		instance. zip is not supported for python2.2.
	
	undefine|undefine-interpreter
		
		Removes the given interpreter from the dictionaries
	
	update|update-plist
	
		Updates the %(plistInfo)s based on the current pickle dictionary, 
		printing an error if it was not possible.

Files,
	(Project Directory)/%(interpDir)s/%(pickleDict)s (generated)
	(Project Directory)/%(interpDir)s/%(plistInfo)s (generated)
	(Project Directory)/Resources/TomatoTorrentAgent/
	
	%(pickleDict)s is used by the project build scripts to compile
	optimized BitTorrent python code. %(plistInfo)s is used by the
	BitTorrent Macintosh app to determine what optimized BitTorrent
	code it has (if it doesn't have one for the user's python, it 
	defaults to a source-code based zip file that req's at least 
	py2.3).
	
	BitTorrentAgent/ is populated during build, and then copied
	into the application's bundle resources.

"""

def interpreters_folder():
	return os.path.join(os.path.join(os.getcwd(), kRootFolder), kInterpretersFolder)

def resources_folder():
	return os.path.join(os.path.join(os.getcwd(), kRootFolder), kResourcesFolder)

def key_for_current_interp():
	return 'python%d.%d' % sys.version_info[:2]


def get_dictionary():
	result = {}
	try:
		foo = file(os.path.join(interpreters_folder(), kPickleDictionaryName), 'rb')
		result = pickle.load(foo)
		foo.close()
	except:
		pass
			
	return result

def update_plist(bar):
	info = None
	plistFilePath = os.path.join(resources_folder(), kPythonInfoPlistName)
	
	# the only key we want is zipimport.
	foo = {}
	for x in bar.keys():
		foo.setdefault(x, {})['zipimport'] = bar[x]['zipimport']
		
	try:
		import plistservices

		try:
			plistservices.dataFromPropertyList(foo).writeToFile(plistFilePath)
		except:
			info = (str(sys.exc_info()[0]), str(sys.exc_info()[1]))
			
	except ImportError:
		try:
			import plistlib
			
			try:
				bar = plistlib.Plist(**foo).write(plistFilePath)
			except:
				info = (str(sys.exc_info()[0]), str(sys.exc_info()[1]))
				
		except ImportError:
			info = "Neither plistservices or plistlib is available."
	
	if not info: info = "Successfully updated %s" % plistFilePath
	
	return str(info)

def save_dictionary(foo):
	try:
		bar = file(os.path.join(interpreters_folder(), kPickleDictionaryName), 'wb')
		pickle.dump(foo, bar)
		bar.close()
		
		update_plist(foo)
		
	except pickle.PickleError, e:
		print >>sys.stderr, str(e)

def run_define():

	zipimport = 1
	if len(sys.argv) == 3 and sys.argv[2] == '--no-zip':
		zipimport = 0
	
	foo = get_dictionary()
	
	foo[key_for_current_interp()] = {
		'sys.executable' 		: sys.executable,
		'zipimport'				: zipimport,
		'optimize'				: 1
	}
	
	save_dictionary(foo)

	print "Defined %s with zipimport: %d" % (key_for_current_interp(), zipimport)

def run_undef():
	foo = get_dictionary()
	key = key_for_current_interp()
	if key in foo:
		del(foo[key])
		save_dictionary(foo)
		
		print "Did undefine %s" % key
	else:
		print "%s was not defined." % key

def run_update():
	foo = get_dictionary()
	print update_plist(foo)

if __name__ == '__main__':
	pystuff = {}

	pystuff['command'] = sys.argv[0]
	pystuff['pickleDict'] = kPickleDictionaryName
	pystuff['plistInfo'] = kPythonInfoPlistName
	pystuff['interpDir'] = kInterpretersFolder
	
	commands = {
		'define-interpreter' 	: run_define,
		'undefine-interpreter'	: run_undef,
		'update-plist'			: run_update 
	}
	
	printUsage = 1
	if len(sys.argv) >= 2:
	
		if len(glob.glob("*.xcode")) > 0:
			
			if not os.path.exists(kInterpretersFolder):
				os.mkdir(kInterpretersFolder)
			if not os.path.exists(os.path.join('Resources', 'TomatoTorrentAgent')):
				try: os.mkdir(os.path.join('Resources', 'TomatoTorrentAgent'))
				except: pass
				
			command = fnmatch.filter(commands.keys(), sys.argv[1] + '*')
			
			if len(command) == 1:
				command = command[0]
				printUsage = None
				commands[command]()
				
		else:
			print >>sys.stderr,	"*** There's no project here!\n"
	
	if printUsage:
		print description % pystuff
	
		
	
