import sys, os, commands, compileall
from glob import glob
from DefinePythonInterpreters import get_dictionary

def run_shell_script(command):
	"""Runs the given shell script lines with zsh, returning the error code if any.
	
	Output is printed after script has run; stdout first, then stderr (with Error:
	prefixed to each line of stderr). """
	
	shellin, shellout, shellerr = os.popen3("zsh")
	
	print command
	shellin.write(command + '\n')
	shellin.flush()
	shellin.close()
	
	commandOutput = shellout.read()
	if len(commandOutput): print commandOutput
	shellout.close()
	
	errors = shellerr.readlines()
	if len(errors):
		errors = ['Error: ' + x for x in errors]
		for error in errors: print error

	shellerr.close()


if __name__ == '__main__':

	if len(sys.argv) > 1 and os.path.isdir(sys.argv[1]):
		foo = get_dictionary()
		source = os.getcwd()
		
		### get files not in project, and just relative to source.
		nonProjFiles = []
		output = commands.getoutput('/Developer/Tools/pbutil -diskToProject')
		sourcePlusSlash = len(source) + 1
		nonProjFiles = [path.strip()[sourcePlusSlash:] for path in output.split('\n')[2:-1]] 
	
		### source files to copy. each of the items in this var is used as the first argument to pbxcp,
		# as in pbxcp sourceFiles[i] destFolder. *.py is used to filter unwanted files in a dir.
		sourceFiles = []
		sourceFiles.extend([('', x) for x in glob('TomatoTorrentAgent/Python/*.py') if x not in nonProjFiles])
		sourceFiles.extend([('', x) for x in glob('TomatoTorrentAgent/Python/TomatoTorrent') if x not in nonProjFiles])
		sourceFiles.extend([('BitTorrent', x) for x in glob('../bitconjurer-4/BitTorrent/*.py')])
		sourceFiles.extend([('khashmir', x) for x in glob('../bitconjurer-4/khashmir/*.py')])
		sourceFiles.append(('', '../bitconjurer-4/LICENSE.txt'))
		print >>sys.stderr, '=> all source files:'
		for x in sourceFiles: print >>sys.stderr, '\t', os.path.join(*x)
		
		### filter files I don't want.
		junkFiles = 'TorrentQueue', 'GUI', 'testtest', 'controlsocket', 'Desktop', 'OpenPath', 'launchmanycore'
		junkFiles = ['../bitconjurer-4/BitTorrent/' + x + '.py' for x in junkFiles]
		junkFiles = [('BitTorrent', x) for x in junkFiles]
		print >>sys.stderr, ""
		print >>sys.stderr, '=> all junk files:'
		for x in junkFiles: print >>sys.stderr, '\t', x
		
		sourceFiles = [x for x in sourceFiles if x not in junkFiles]
		print >>sys.stderr, ""
		print >>sys.stderr, '=> files to install:'
		for x in sourceFiles: print >>sys.stderr, '\t', os.path.join(*x)
		
		### destination folder
		destFolder = sys.argv[1] #'BitTorrentAgent/Resources/BitTorrentAgent'
		
		# always define this key to this value.
		foo['pythonZip'] = {'zipimport':1, 'optimize':0}
		
		for key in foo:
			print key
			print '\t', foo[key]
			
			currentDestFolder = os.path.join(destFolder, key)
			if not os.path.exists(currentDestFolder): 
				os.mkdir(currentDestFolder)
			else:
				command = """
					/bin/rm -rf "%(currentDestFolder)s"
				""" % {'currentDestFolder':currentDestFolder}
				run_shell_script(command)
	
				# make it again
				os.mkdir(currentDestFolder)
				
			for item in sourceFiles:
				
				copyDestFolder = os.path.join(currentDestFolder, item[0])
				if not os.path.exists(copyDestFolder): os.mkdir(copyDestFolder)
				
				command = '/Developer/Tools/pbxcp -exclude .DS_Store -exclude CVS -exclude .svn "%s" "%s"' \
					% (item[1], copyDestFolder)
				
				status = run_shell_script(command)
				### if any commands fail, this thing aborts with that exit code.
				if status is not None: sys.exit(result)
			
			if os.environ.get("SK_TomatoTorrentAgent_PythonOptimized", "0") == "1" and foo[key].get('optimize',1) == 1:
				print >>sys.stderr, ""
				print >>sys.stderr, "\n=> Doing optimized install"
				command = """
				
					cd "%(currentDestFolder)s"
					%(python)s -OO -c "import compileall; compileall.compile_dir('.')"
					/bin/rm -vf **/*.p(y|yc)
				
				""" % {'currentDestFolder':currentDestFolder, 'python': foo[key]['sys.executable']}
	
				status = run_shell_script(command)
				if status is not None: sys.exit(result) 
			else:
				print >>sys.stderr, ""
				print >>sys.stderr, "\n=> NOT doing optimized install"

				# if we're not optimizing, let's clean any *.pyc, *.pyo that may be around.	
				command = """
					cd "%(currentDestFolder)s"
					/bin/rm -vf **/*.py[oc]
				""" % {'currentDestFolder':currentDestFolder}
	
				status = run_shell_script(command)
				#if status is not None: sys.exit(result)
				#	==> don't care if no matches
	
			if foo[key]['zipimport']:
				# and in this case we zip up whatever work we did.
				command = """
					if [ -d "%(currentDestFolder)s" ] 
					then
						cd "%(currentDestFolder)s"
						/usr/bin/zip -r -m ttagent.zip *
					fi
				""" % {'currentDestFolder':currentDestFolder}
				
				status = run_shell_script(command)
				if status is not None: sys.exit(result)
	else:
		print >>sys.stderr, """
Usage: %s <install directory path>

The first argument to the script must be a directory to install the sources into.
If the SK_TomatoTorrentAgent_PythonOptimized environment variable is set to "1", python will be executed with -OO.""" % sys.argv[0]
		sys.exit(1)
