//==============================================================================
//		• ttagent_posix_cf.m
//==============================================================================
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khan’s name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//	code that relies on posix (popen) and corefoundation.
//

#include "tomato_agent.h"
#include "tomato_agent_logging.h"

#include <stdio.h>
#include <stdbool.h>
#include <CoreFoundation/CoreFoundation.h>

#define k_pyscript_pythonversion " -c \"import sys; print 'python%d.%d' % sys.version_info[:2]\""

char g_ttagent_python_sourcedir[32] 	= k_ttagent_python_sourcedir_generic;	//python2.x, pythonZip
bool g_ttagent_python_zipimport 		= true;


void ttagent_initialize(const char* pythonInfoPath)
{
	//get the version of python installed on this machine.
	FILE		*output;
	long		scanfResult;
	char		buffer[256];

	#ifdef _tomatotorrentagent_py_optimized_
		//use pyo files instead of pyc
		setenv("PYTHONOPTIMIZE", "1", 1);
	#endif
		
	if (output = popen(k_ttagent_python_path k_pyscript_pythonversion, "r"))
	{
		scanfResult = fscanf(output, "%255s\n", buffer);
		if (pclose(output) == 0 && scanfResult == 1)
		{
			//check the plist, see if we have this version defined.
			CFStringRef		infoKey;
			CFStringRef		xmlPath;
			CFURLRef		xmlURL;
			CFDataRef		xmlData;
			CFDictionaryRef	plist;
			CFDictionaryRef	versionInfo;
			CFNumberRef		zipImportInfo;
			
			if (xmlPath = CFStringCreateWithCString(NULL, pythonInfoPath, kCFStringEncodingUTF8))
			{
				if (xmlURL = CFURLCreateWithFileSystemPath(NULL, xmlPath, kCFURLPOSIXPathStyle, false))
				{
					if (CFURLCreateDataAndPropertiesFromResource(NULL, xmlURL, &xmlData, NULL, NULL, NULL) == true)
					{
						if (plist = CFPropertyListCreateFromXMLData(NULL, xmlData, kCFPropertyListImmutable, NULL))
						{
							//good. we loaded the dictionary. now make a CFString out of 'buffer',
							if (infoKey = CFStringCreateWithCStringNoCopy(NULL, buffer, kCFStringEncodingASCII, kCFAllocatorNull))
							{
								if (versionInfo = CFDictionaryGetValue(plist, infoKey))
								{
									//we have an optimization or special case for this version of python.
									strncpy(g_ttagent_python_sourcedir, buffer, sizeof(g_ttagent_python_sourcedir)-1);
									
									if (zipImportInfo = CFDictionaryGetValue(versionInfo, CFSTR("zipimport")))
									{
										long		foo;
										
										if (CFNumberGetValue(zipImportInfo, kCFNumberLongType, &foo))
										{
											g_ttagent_python_zipimport = foo;
										}
									}
								}
								//else use default settings.
								
								CFRelease(infoKey);
							}
							
							CFRelease(plist);
						}
						
						CFRelease(xmlData);
					}
					
					CFRelease(xmlURL);
				}
				
				CFRelease(xmlPath);
			}
		}
	}
	
	ttagent_logging_debug(0, "ttagent_initialize: sourcedir = %s, zipimport = %hd, optimize = %s.",
		g_ttagent_python_sourcedir, 
		(short)g_ttagent_python_zipimport,
		
		#ifdef _tomatotorrentagent_py_optimized_
			"yes"
		#else
			"no"
		#endif
	);
}

